import doctest
from insights.parsers import udev_rules
from insights.parsers.udev_rules import UdevRulesFCWWPN
from insights.tests import context_wrap

UDEV_RULES_FILT_HIT = """
ENV{FC_TARGET_WWPN}!="$*"; GOTO="fc_wwpn_end"
ENV{FC_INITIATOR_WWPN}!="$*"; GOTO="fc_wwpn_end"
ENV{FC_TARGET_LUN}!="$*"; GOTO="fc_wwpn_end"
""".strip()


def test_documentation():
    env = {'udev_rules': UdevRulesFCWWPN(context_wrap(UDEV_RULES_FILT_HIT))}
    failed_count, tests = doctest.testmod(udev_rules, globs=env)
    assert failed_count == 0


def test_udev_rules():
    result = UdevRulesFCWWPN(context_wrap(UDEV_RULES_FILT_HIT))
    for line in ['ENV{FC_TARGET_WWPN}!="$*"; GOTO="fc_wwpn_end"',
                 'ENV{FC_INITIATOR_WWPN}!="$*"; GOTO="fc_wwpn_end"',
                  'ENV{FC_TARGET_LUN}!="$*"; GOTO="fc_wwpn_end"']:
        assert line in result.lines



import doctest
import pytest

from insights.parsers import config_file_perms
from insights.parsers.config_file_perms import SshdConfigPerms, Grub1ConfigPerms, Grub2ConfigPerms
from insights.tests import context_wrap


PARSERS = [
    # (parser class, path to configuration file)
    (SshdConfigPerms, "/etc/ssh/sshd_config"),
    (Grub1ConfigPerms, "/boot/grub/grub.conf"),
    (Grub2ConfigPerms, "/boot/grub2/grub.cfg")
]

TEST_CASES_PERMISSIONS = [
    # (line, owned_by_root, only_root_can_write)
    ('-rw-rw-r--. 1 root root 4179 Dec  1  2014 ', True, True),
    ('-rw-r--rw-. 1 root root 4179 Dec  1  2014 ', True, False),
    ('-rw-r--r--. 1 root user 4179 Dec  1  2014 ', False, True),
    ('-rw-rw-r--. 1 root user 4179 Dec  1  2014 ', False, False),
    ('-rw-r--r--. 1 user root 4179 Dec  1  2014 ', False, False),
]


@pytest.mark.parametrize("parser, path", PARSERS)
@pytest.mark.parametrize("line, owned_by_root, only_root_can_write", TEST_CASES_PERMISSIONS)
def test_sshd_grub(parser, path, line, owned_by_root, only_root_can_write):
    line_with_path = line + path
    result = parser(context_wrap(line_with_path))
    assert result.line == line_with_path
    assert result.owned_by("root", also_check_group=True) == owned_by_root
    assert result.only_root_can_write() == only_root_can_write


def test_doc_examples():
    sshd = "-rw-------. 1 root root 4179 Dec  1  2014 /etc/ssh/sshd_config"
    grub1 = "-rw-r--r--. 1 root root 4179 Dec  1  2014 /boot/grub/grub.conf"
    grub2 = "-rw-r--r--. 1 root root 4179 Dec  1  2014 /boot/grub2/grub.cfg"

    env = {
        "sshd_perms": SshdConfigPerms(context_wrap(sshd)),
        "grub1_perms": Grub1ConfigPerms(context_wrap(grub1)),
        "grub2_perms": Grub2ConfigPerms(context_wrap(grub2)),
    }
    failed, total = doctest.testmod(config_file_perms, globs=env)
    assert failed == 0



"""
Satellite Version
=================

The following modules are included:

SatelliteVersion - Version of Satellite Server
----------------------------------------------
Combiner to get Satellite Server version information.

CapsuleVersion - Version of Satellite Capsule (>=6.2)
-----------------------------------------------------
Combiner to get Satellite Capsule version information. ONLY Satellite Capsule
6.2 and newer are supported.


"""

from insights import combiner, SkipComponent
from insights.parsers.satellite_version import Satellite6Version as Sat6Ver
from insights.parsers.installed_rpms import InstalledRpms


# NOTE:
# The following table only tracks 6.0.x and 6.1.x.
# See https://access.redhat.com/articles/1343683
# But, there are some mistakes in the KCS, the versions in below map
# are corrected according to the corresponding ERRATA pages.
#
# Update: Thu Nov 14 10:51:19 CST 2019
#
sat6_ver_map = {
        # Sat     foreman     candlepin   katello
        '6.0.8': ('1.6.0.53', '0.9.23', '1.5.0'),
        '6.1.1': ('1.7.2.33', '0.9.49.3', '2.2.0.14'),
        '6.1.2': ('1.7.2.36', '0.9.49.6', '2.2.0.16'),
        '6.1.3': ('1.7.2.43', '0.9.49.8', '2.2.0.16'),
        '6.1.4': ('1.7.2.46', '0.9.49.9', '2.2.0.16'),
        '6.1.5': ('1.7.2.49', '0.9.49.9', '2.2.0.16'),
        '6.1.6': ('1.7.2.50', '0.9.49.9', '2.2.0.17'),
        '6.1.7': ('1.7.2.53', '0.9.49.11', '2.2.0.17'),
        '6.1.8': ('1.7.2.55', '0.9.49.12', '2.2.0.19'),
        '6.1.9': ('1.7.2.56', '0.9.49.12', '2.2.0.19'),
        '6.1.10': ('1.7.2.61', '0.9.49.16', '2.2.0.19'),
        '6.1.11': ('1.7.2.62', '0.9.49.19', '2.2.0.19'),
        '6.1.12': ('1.7.2.63', '0.9.49.23', '2.2.0.19'),
}


def _parse_sat_version(version):
    ver_sp = version.split(".") if version else []
    major = int(ver_sp[0]) if ver_sp and ver_sp[0].isdigit() else None
    minor = int(ver_sp[1]) if len(ver_sp) > 1 and ver_sp[1].isdigit() else None
    return [major, minor]


@combiner(InstalledRpms, optional=[Sat6Ver])
class SatelliteVersion(object):
    """
    Check the parsers
    :class:`insights.parsers.satellite_version.Satellite6Version` and
    :class:`insights.parsers.installed_rpms.InstalledRpms` for satellite version
    information.

    Below is the logic to determine the satellite version::

        1. For Satellite 6.1:

            a. Check the version information in below files at first
               - https://access.redhat.com/solutions/1392633
               File: /usr/share/foreman/lib/satellite/version.rb

            b. Check the version of package foreman, candlepin and katello, E.g.
               - https://access.redhat.com/articles/1343683

                            Sat 6.0.8   Sat 6.1.10  Sat 6.1.11
               foreman      1.6.0.53    1.7.2.61    1.7.2.62
               candlepin    0.9.23      0.9.49.16   0.9.49.19
               katello      1.5.0       2.2.0       2.2.0

        2. For Satellite 6.2 and newer:

           Check the version of satellite package directly:
           - https://access.redhat.com/solutions/1392633

                                    Sat 6.0.x   Sat 6.1.x   Sat 6.2.x
                satellite           -           -           6.2.x

        3. For Satellite 5.x
           - https://access.redhat.com/solutions/1224043
             NOTE: Because of satellite-branding is not deployed in Satellite
                   5.0~5.2, and satellite-schema can also be used for checking
                   the version, here checked satellite-schema instead of
                   satellite-branding.

           Check the version of package satellite-schema directly:

                                        Sat 5.0~5.2     Sat 5.3 ~
                rhn-satellite-schema    ok              -
                satellite-schema        -               ok

    Attributes:
        full(str): the full version format like `version-release`.
        version(str): the satellite version do not includes `release`.
        release(str): the `release` string in the version.
        major(int): the major version.
        minor(int): the minor version.

    Raises:
        SkipComponent: When it's not a Satellite machine or the Satellite
            version cannot be determined according to current information.

    Examples:
        >>> sat_ver.full == 'satellite-6.2.0.11-1.el7sat'
        True
        >>> sat_ver.major
        6
        >>> sat_ver.minor
        2
        >>> sat_ver.version
        '6.2.0.11'
        >>> sat_ver.release
        '1.el7sat'
    """
    def __init__(self, rpms, sat6_ver):
        self.full = None
        self.version = None
        self.release = None
        self.major = None
        self.minor = None

        # For Satellite 6.1, if satellite_version/version.rb is available:
        if sat6_ver:
            # no 'release' in this case, but more accurate
            self.full = sat6_ver.full
            self.version = sat6_ver.version
            self.major = sat6_ver.major
            self.minor = sat6_ver.minor
        else:
            # For Satellite 6.2 and newer, check the satellite package directly
            sat62_pkg = rpms.get_max('satellite')
            if sat62_pkg:
                self.full = sat62_pkg.package
                self.version = sat62_pkg.version
                self.release = sat62_pkg.release
                self.major, self.minor = _parse_sat_version(self.version)
            else:
                # For Satellite 6.0/6.1, check the version of:
                # - foreman, candlepin and katello
                fman = rpms.get_max('foreman')
                cndp = rpms.get_max('candlepin')
                ktlo = rpms.get_max('katello')
                if fman and cndp and ktlo:
                    for sat_ver, map_ver in sat6_ver_map.items():
                        if all(pkg.version.startswith(mv) for pkg, mv in zip([fman, cndp, ktlo], map_ver)):
                            # no 'release' in this situation
                            self.major, self.minor = _parse_sat_version(sat_ver)
                            self.full = self.version = sat_ver
                else:
                    # For Satellite 5.x
                    sat5_pkg = rpms.get_max('satellite-schema') or rpms.get_max('rhn-satellite-schema')
                    if sat5_pkg:
                        self.full = sat5_pkg.package
                        self.version = sat5_pkg.version
                        self.release = sat5_pkg.release
                        self.major, self.minor = _parse_sat_version(self.version)
        if not self.full:
            raise SkipComponent("Not a Satellite machine or unable to determine Satellite version")


@combiner(InstalledRpms)
class CapsuleVersion(object):
    """
    Check the parser
    :class:`insights.parsers.installed_rpms.InstalledRpms` for satellite capsule
    version information.

    .. note::
        ONLY Satellite Capsule 6.2 and newer are supported.

    Below is the logic to determine the satellite version::

        Check the version of satellite/satellite-capsule directly:
        - https://access.redhat.com/solutions/1392633

                                Sat 6.0.x   Sat 6.1.x   Sat 6.2.x
            satellite-capsule   -           -           6.2.x

    Attributes:
        full(str): the full version format like `version-release`.
        version(str): the satellite version do not includes `release`.
        release(str): the `release` string in the version.
        major(int): the major version.
        minor(int): the minor version.

    Raises:
        SkipComponent: When it's not a Satellite Capsule machine or the
            Satellite Capsule version cannot be determined according to
            current information.

    Examples:
        >>> cap_ver.full == 'satellite-capsule-6.2.0.11-1.el7sat'
        True
        >>> cap_ver.major
        6
        >>> cap_ver.minor
        2
        >>> cap_ver.version
        '6.2.0.11'
        >>> cap_ver.release
        '1.el7sat'
    """
    def __init__(self, rpms):
        self.full = None
        self.version = None
        self.release = None
        self.major = None
        self.minor = None

        # For Capsule, ONLY 6.2 and newer are supported
        sat62_pkg = rpms.get_max('satellite-capsule')
        # foreman package should not be there on Capsule Server
        if sat62_pkg and 'foreman' not in rpms:
            self.full = sat62_pkg.package
            self.version = sat62_pkg.version
            self.release = sat62_pkg.release
            self.major, self.minor = _parse_sat_version(self.version)
        else:
            raise SkipComponent("Not a Satellite Capsule machine or unable to determine the version")



from insights.parsers import httpd_V
from insights.parsers import SkipException
from insights.parsers.httpd_V import HttpdV
from insights.tests import context_wrap
import pytest
import doctest

HTTPD_V_22 = """
Server version: Apache/2.2.15 (Unix)
Server built:   Feb  4 2016 02:44:09
Server's Module Magic Number: 20051115:25
Server loaded:  APR 1.3.9, APR-Util 1.3.9
Compiled using: APR 1.3.9, APR-Util 1.3.9
Architecture:   64-bit
Server MPM:     Prefork
  threaded:     no
    forked:     yes (variable process count)
Server compiled with....
 -D APACHE_MPM_DIR="server/mpm/prefork"
 -D APR_HAS_SENDFILE
 -D APR_HAS_MMAP
 -D APR_HAVE_IPV6 (IPv4-mapped addresses enabled)
 -D APR_USE_SYSVSEM_SERIALIZE
 -D APR_USE_PTHREAD_SERIALIZE
 -D SINGLE_LISTEN_UNSERIALIZED_ACCEPT
 -D APR_HAS_OTHER_CHILD
 -D AP_HAVE_RELIABLE_PIPED_LOGS
 -D DYNAMIC_MODULE_LIMIT=128
 -D HTTPD_ROOT="/etc/httpd"
 -D SUEXEC_BIN="/usr/sbin/suexec"
 -D DEFAULT_PIDLOG="run/httpd.pid"
 -D DEFAULT_SCOREBOARD="logs/apache_runtime_status"
 -D DEFAULT_LOCKFILE="logs/accept.lock"
 -D DEFAULT_ERRORLOG="logs/error_log"
 -D AP_TYPES_CONFIG_FILE="conf/mime.types"
 -D SERVER_CONFIG_FILE="conf/httpd.conf"
""".strip()

HTTPD_V_24 = """
Server version: Apache/2.4.6 (Red Hat Enterprise Linux)
Server built:   Aug  3 2016 08:33:27
Server's Module Magic Number: 20120211:24
Server loaded:  APR 1.4.8, APR-UTIL 1.5.2
Compiled using: APR 1.4.8, APR-UTIL 1.5.2
Architecture:   64-bit
Server MPM:     Worker
  threaded:     no
    forked:     yes (variable process count)
Server compiled with....
 -D APR_HAS_SENDFILE
 -D APR_HAS_MMAP
 -D APR_HAVE_IPV6 (IPv4-mapped addresses enabled)
 -D APR_USE_SYSVSEM_SERIALIZE
 -D APR_USE_PTHREAD_SERIALIZE
 -D SINGLE_LISTEN_UNSERIALIZED_ACCEPT
 -D APR_HAS_OTHER_CHILD
 -D AP_HAVE_RELIABLE_PIPED_LOGS
 -D DYNAMIC_MODULE_LIMIT=256
 -D HTTPD_ROOT="/etc/httpd"
 -D SUEXEC_BIN="/usr/sbin/suexec"
 -D DEFAULT_PIDLOG="/run/httpd/httpd.pid"
 -D DEFAULT_SCOREBOARD="logs/apache_runtime_status"
 -D DEFAULT_ERRORLOG="logs/error_log"
 -D AP_TYPES_CONFIG_FILE="conf/mime.types"
 -D SERVER_CONFIG_FILE="conf/httpd.conf"
""".strip()


HTTPDV_DOC = """
Server version: Apache/2.2.6 (Red Hat Enterprise Linux)
Server's Module Magic Number: 20120211:24
Compiled using: APR 1.4.8, APR-UTIL 1.5.2
Architecture:   64-bit
Server MPM:     Prefork
Server compiled with....
-D APR_HAS_SENDFILE
-D APR_HAVE_IPV6 (IPv4-mapped addresses enabled)
-D AP_TYPES_CONFIG_FILE="conf/mime.types"
-D SERVER_CONFIG_FILE="conf/httpd.conf"
""".strip()


def test_httpd_V():
    result = HttpdV(context_wrap(HTTPD_V_22, path='/usr/sbin/httpd_-V'))
    assert result["Server MPM"] == "prefork"
    assert result["Server version"] == "apache/2.2.15 (unix)"
    assert result["forked"] == "yes (variable process count)"
    assert "APR_HAVE_IPV6" in result['Server compiled with']
    assert result['Server compiled with']['APR_HAS_MMAP'] is True
    assert result['Server compiled with']['APR_HAVE_IPV6'] == "IPv4-mapped addresses enabled"
    assert result['Server compiled with']['DEFAULT_PIDLOG'] == "run/httpd.pid"
    assert result.httpd_command == "/usr/sbin/httpd"
    assert result.mpm == "prefork"
    assert result.version == "apache/2.2.15 (unix)"

    result = HttpdV(context_wrap(HTTPD_V_24, path='/usr/sbin/httpd.worker_-V'))
    assert result["Server MPM"] == "worker"
    assert result["Server version"] == "apache/2.4.6 (red hat enterprise linux)"
    assert result["forked"] == "yes (variable process count)"
    assert "APR_HAVE_IPV6" in result['Server compiled with']
    assert result['Server compiled with']['APR_HAS_MMAP'] is True
    assert result['Server compiled with']['APR_HAVE_IPV6'] == "IPv4-mapped addresses enabled"
    assert result['Server compiled with']['DEFAULT_PIDLOG'] == "/run/httpd/httpd.pid"
    assert result.httpd_command == "/usr/sbin/httpd.worker"
    assert result.mpm == "worker"
    assert result.version == "apache/2.4.6 (red hat enterprise linux)"


def test_httpd_V_exp():
    with pytest.raises(SkipException) as sc:
        HttpdV(context_wrap(""))
    assert "Input content is empty" in str(sc)

    with pytest.raises(SkipException) as sc:
        HttpdV(context_wrap("TEST"))
    assert "Input content is not empty but there is no useful parsed data." in str(sc)


def test_httpd_V_doc():
    env = {
            'hv': HttpdV(context_wrap(HTTPDV_DOC))
          }
    failed, total = doctest.testmod(httpd_V, globs=env)
    assert failed == 0



from insights.parsers.teamdctl_state_dump import TeamdctlStateDump
from insights.tests import context_wrap

TEAMDCTL_STATE_DUMP_INFO = """
{
    "runner": {
        "active_port": "eno1"
    },
    "setup": {
        "daemonized": false,
        "dbus_enabled": true,
        "debug_level": 0,
        "kernel_team_mode_name": "activebackup",
        "pid": 4464,
        "pid_file": "/var/run/teamd/team0.pid",
        "runner_name": "activebackup",
        "zmq_enabled": false
    },
    "team_device": {
        "ifinfo": {
            "dev_addr": "2c:59:e5:47:a9:04",
            "dev_addr_len": 6,
            "ifindex": 5,
            "ifname": "team0"
        }
    }
}
""".strip()


TEAMDCTL_STATE_DUMP_INFO_NONE = """
{
    "runner": {
    },
    "setup": {
        "daemonized": false,
        "dbus_enabled": true,
        "debug_level": 0,
        "kernel_team_mode_name": "activebackup",
        "pid": 4464,
        "pid_file": "/var/run/teamd/team0.pid",
        "runner_name": "activebackup",
        "zmq_enabled": false
    },
    "team_device": {
        "ifinfo": {
            "dev_addr": "2c:59:e5:47:a9:04",
            "dev_addr_len": 6,
            "ifindex": 5
        }
    }
}
""".strip()


def test_teamdctl_state_dump():
    result = TeamdctlStateDump(context_wrap(TEAMDCTL_STATE_DUMP_INFO))

    assert result.data == {
        'runner': {
            'active_port': 'eno1'
        },
        'setup': {
            'daemonized': False,
            'zmq_enabled': False,
            'kernel_team_mode_name': 'activebackup',
            'pid': 4464,
            'dbus_enabled': True,
            'debug_level': 0,
            'pid_file': '/var/run/teamd/team0.pid',
            'runner_name': 'activebackup'
        },
        'team_device': {
            'ifinfo': {
                'ifindex': 5,
                'dev_addr': '2c:59:e5:47:a9:04',
                'ifname': 'team0',
                'dev_addr_len': 6}
        }
    }
    assert result['runner']['active_port'] == 'eno1'
    assert result['setup']['runner_name'] == 'activebackup'
    assert result.runner_type == 'activebackup'
    assert result.team_ifname == 'team0'


def test_teamdctl_state_dump_none():
    result = TeamdctlStateDump(context_wrap(TEAMDCTL_STATE_DUMP_INFO_NONE))

    assert result.data == {
        'runner': {
        },
        'setup': {
            'daemonized': False,
            'zmq_enabled': False,
            'kernel_team_mode_name': 'activebackup',
            'pid': 4464,
            'dbus_enabled': True,
            'debug_level': 0,
            'pid_file': '/var/run/teamd/team0.pid',
            'runner_name': 'activebackup'
        },
        'team_device': {
            'ifinfo': {
                'ifindex': 5,
                'dev_addr': '2c:59:e5:47:a9:04',
                'dev_addr_len': 6}
        }
    }
    assert result['setup']['runner_name'] == 'activebackup'
    assert result.runner_type == 'activebackup'
    assert result.team_ifname is None



from insights.parsers import SkipException
from insights.tests import context_wrap
from insights.parsers import cpu_vulns
from insights.parsers.cpu_vulns import CpuVulns
import pytest
import doctest

INPUT_MELTDOWN = """
Mitigation: PTI
""".strip()

INPUT_SPECTRE_V1 = """
Mitigation: Load fences
""".strip()

INPUT_SPECTRE_V2_RHEL_7_1 = """
Mitigation: Full generic retpoline, IBPB: conditional, IBRS_FW, STIBP: conditional, RSB filling
""".strip()

INPUT_SPECTRE_V2_RHEL_7 = """
Vulnerable: Retpoline without IBPB
""".strip()

INPUT_SPECTRE_V2_RHEL_6 = """
Mitigation: IBRS (kernel)
""".strip()

INPUT_SPEC_STORE_BYPASS = """
Mitigation: Speculative Store Bypass disabled
""".strip()

INPUT_SPEC_STORE_BYPASS_2 = """
Not affected
""".strip()

INPUT_SPEC_STORE_BYPASS_3 = """
Vulnerable
""".strip()

INPUT_SMT = """
Mitigation: PTE Inversion; VMX: conditional cache flushes, SMT vulnerable
""".strip()

# This could occur if you've not updated to the latest intel-microcode package or if Intel has not released new microcode for your processor. You'll see the following in this situation:
INPUT_MDS = """
Vulnerable: Clear CPU buffers attempted, no microcode; SMT vulnerable
""".strip()

# Processors that aren't vulnerable to MDS will report the following:
INPUT_MDS_2 = """
Not affected
""".strip()

# Processors that have Hyper-Threading support enabled will indicate that SMT is vulnerable:
INPUT_MDS_3 = """
Mitigation: Clear CPU buffers; SMT vulnerable
""".strip()

# The file will contain the following contents for processors that do not support Intel Hyper-Threading or where Hyper-Threading has been disabled:
INPUT_MDS_4 = """
Mitigation: Clear CPU buffers; SMT disabled
""".strip()

# The kernel is unable to reliably determine whether Hyper-Threading is enabled when running in a virtual environment. Updated host kernel packages, updated host qemu packages with proper configuration to pass through the host CPU type to the guest, and updated guest kernel packages will show the following status inside of the virtual environment:
INPUT_MDS_5 = """
Mitigation: Clear CPU buffers; SMT Host state unknown
""".strip()


# CpuVulnsMeltdown
def test_cpu_vulns_meltdown():
    spectre = CpuVulns(context_wrap(INPUT_MELTDOWN, path='/sys/devices/system/cpu/vulnerabilities/meltdown'))
    assert spectre.value == INPUT_MELTDOWN
    assert spectre.file_name == 'meltdown'


def test_cpu_vulns_meltdown_exp1():
    """
    Here test the examples cause expections
    """
    with pytest.raises(SkipException) as sc1:
        CpuVulns(context_wrap('', path='/sys/devices/system/cpu/vulnerabilities/meltdown'))
    assert "Input content is empty" in str(sc1)


# CpuVulnsSpecStoreBypass
def test_cpu_vulns_spec_store_bypass():
    spectre = CpuVulns(context_wrap(INPUT_SPEC_STORE_BYPASS, path='/sys/devices/system/cpu/vulnerabilities/spec_store_bypass'))
    assert spectre.value == INPUT_SPEC_STORE_BYPASS
    assert spectre.file_name == 'spec_store_bypass'


def test_cpu_vulns_spec_store_bypass_2():
    spectre = CpuVulns(context_wrap(INPUT_SPEC_STORE_BYPASS_2, path='/sys/devices/system/cpu/vulnerabilities/spec_store_bypass'))
    assert spectre.value == INPUT_SPEC_STORE_BYPASS_2
    assert spectre.file_name == 'spec_store_bypass'


def test_cpu_vulns_spec_store_bypass_3():
    spectre = CpuVulns(context_wrap(INPUT_SPEC_STORE_BYPASS_3, path='/sys/devices/system/cpu/vulnerabilities/spec_store_bypass'))
    assert spectre.value == INPUT_SPEC_STORE_BYPASS_3
    assert spectre.file_name == 'spec_store_bypass'


def test_cpu_vulns_spec_store_bypass_exp1():
    """
    Here test the examples cause expections
    """
    with pytest.raises(SkipException) as sc1:
        CpuVulns(context_wrap('', path='meltdown'))
    assert "Input content is empty" in str(sc1)


# CpuVulnsSpectreV1
def test_cpu_vulns_spectre_v1():
    spectre = CpuVulns(context_wrap(INPUT_SPECTRE_V1, path='/sys/devices/system/cpu/vulnerabilities/spectre_v1'))
    assert spectre.value == INPUT_SPECTRE_V1
    assert spectre.file_name == 'spectre_v1'


def test_cpu_vulns_spectre_v1_exp1():
    """
    Here test the examples cause expections
    """
    with pytest.raises(SkipException) as sc1:
        CpuVulns(context_wrap(''))
    assert "Input content is empty" in str(sc1)


# CpuVulnsSpectreV2
def test_cpu_vulns_spectre_v2_rhel7():
    """
    Here test the examples for spectre_v2
    """
    spectre = CpuVulns(context_wrap(INPUT_SPECTRE_V2_RHEL_7, path='/sys/devices/system/cpu/vulnerabilities/spectre_v2'))
    assert spectre.value == INPUT_SPECTRE_V2_RHEL_7
    assert spectre.file_name == 'spectre_v2'


def test_cpu_vulns_spectre_v2_rhel6():
    """
    Here test the examples for spectre_v2
    """
    spectre = CpuVulns(context_wrap(INPUT_SPECTRE_V2_RHEL_6, path='/sys/devices/system/cpu/vulnerabilities/spectre_v2'))
    assert spectre.value == INPUT_SPECTRE_V2_RHEL_6
    assert spectre.file_name == 'spectre_v2'


def test_cpu_vulns_spectre_v2_exp1():
    """
    Here test the examples cause expections
    """
    with pytest.raises(SkipException) as sc1:
        CpuVulns(context_wrap(''))
    assert "Input content is empty" in str(sc1)


def test_cpu_vulns_documentation():
    """
    Here we test the examples in the documentation automatically using doctest.
    We set up an environment which is similar to what a rule writer might see -
    a '/sys/devices/system/cpu/vulnerabilities/*' output that has been
    passed in as a parameter to the rule declaration.
    """
    env = {
        'sp_v1': CpuVulns(context_wrap(INPUT_SPECTRE_V1,
            path='/sys/devices/system/cpu/vulnerabilities/spectre_v1')),
        'sp_v2': CpuVulns(context_wrap(INPUT_SPECTRE_V2_RHEL_7,
            path='/sys/devices/system/cpu/vulnerabilities/spectre_v2')),
        'md': CpuVulns(context_wrap(INPUT_MELTDOWN,
            path='/sys/devices/system/cpu/vulnerabilities/meltdown')),
        'ssb': CpuVulns(context_wrap(INPUT_SPEC_STORE_BYPASS,
            path='/sys/devices/system/cpu/vulnerabilities/spec_store_bypass'))}

    failed, total = doctest.testmod(cpu_vulns, globs=env)
    assert failed == 0



from insights.core.plugins import make_fail
from insights.plugins import insights_heartbeat
from insights.parsers.hostname import Hostname
from insights.specs import Specs
from insights.tests import context_wrap, InputData, run_test


NON_MATCHING_HOSTNAME = "some-other-hostname-that-doesnt-match"

good = Hostname(context_wrap(insights_heartbeat.HOST))
bad = Hostname(context_wrap(NON_MATCHING_HOSTNAME))


def test_heartbeat():
    expected_result = make_fail(insights_heartbeat.ERROR_KEY)
    assert expected_result == insights_heartbeat.is_insights_heartbeat(good)
    assert insights_heartbeat.is_insights_heartbeat(bad) is None


def test_integration_tests():
    comp = insights_heartbeat.is_insights_heartbeat

    input_data = InputData(name="Match: no kernel")
    input_data.add(Specs.hostname, insights_heartbeat.HOST)
    expected = make_fail(insights_heartbeat.ERROR_KEY)
    run_test(comp, input_data, expected)

    input_data = InputData(name="No Match: bad hostname")
    input_data.add(Specs.hostname, NON_MATCHING_HOSTNAME)
    run_test(comp, input_data, None)



from insights.parsr.iniparser import parse_doc


DATA = """
[DEFAULT]
trooper_guns = miss

[global]
logging=debug
log=/var/logs/sample.log

# Keep this info secret
[secret_stuff]
username=dvader
password=luke_is_my_son

[facts]
major_vulnerability=ray-shielded particle exhaust vent
vader = definitely Luke's
    father

[settings]
music=The Imperial March
color=blue
#blah
color=black
accuracy=0
banks=0 1 2

[novalue]
the_force

""".strip()


def test_iniparser():
    res = parse_doc(DATA, None)
    assert len(res) == 5


def test_hanging_indent():
    res = parse_doc(DATA, None)
    assert res["facts"]["vader"][0].value == "definitely Luke's father"


def test_defaults():
    res = parse_doc(DATA, None)
    assert res["facts"]["trooper_guns"][0].value == "miss"


def test_multiple_values():
    res = parse_doc(DATA, None)
    assert len(res["settings"]["color"]) == 2
    assert res["settings"]["accuracy"][0].value == "0"
    assert res["settings"]["banks"][0].value == "0 1 2"


def test_no_value():
    res = parse_doc(DATA, None)
    assert res["novalue"]["the_force"][0].value is None



from insights.parsers.virt_what import VirtWhat as VWP
from insights.parsers.dmidecode import DMIDecode
from insights.combiners.virt_what import VirtWhat
from insights.tests import context_wrap

T1 = """
kvm
""".strip()

T2 = """

""".strip()

# occasionally we have 2 lines of output
T3 = """
xen
xen-dom0
aws
""".strip()

T4 = """
virt-what: virt-what-cpuid-helper program not found in $PATH
""".strip()

# test for ovirt
T5 = """
ovirt
kvm
""".strip()


DMIDECODE = '''
# dmidecode 2.11
SMBIOS 2.7 present.
188 structures occupying 5463 bytes.
Table at 0xBFFCB000.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: HP
\tVersion: P70
\tRelease Date: 03/01/2013
\tAddress: 0xF0000
\tRuntime Size: 64 kB
\tROM Size: 8192 kB
\tCharacteristics:
\t\tPCI is supported
\t\tPNP is supported
\t\tBIOS is upgradeable
\t\tBIOS shadowing is allowed
\t\tESCD support is available
\t\tBoot from CD is supported
\t\tSelectable boot is supported
\t\tEDD is supported
\t\t5.25"/360 kB floppy services are supported (int 13h)
\t\t5.25"/1.2 MB floppy services are supported (int 13h)
\t\t3.5"/720 kB floppy services are supported (int 13h)
\t\tPrint screen service is supported (int 5h)
\t\tj042 keyboard services are supported (int 9h)
\t\tSerial services are supported (int 14h)
\t\tPrinter services are supported (int 17h)
\t\tCGA/mono video services are supported (int 10h)
\t\tACPI is supported
\t\tUSB legacy is supported
\t\tBIOS boot specification is supported
\t\tFunction key-initiated network boot is supported
\t\tTargeted content distribution is supported
\tFirmware Revision: 1.22

Handle 0x0100, DMI type 1, 27 bytes
System Information
\tManufacturer: HP
\tProduct Name: ProLiant DL380p Gen8
\tVersion: Not Specified
\tSerial Number: 2M23360006
\tUUID: 34373936-3439-4D32-3233-333630303036
\tWake-up Type: Power Switch
\tSKU Number: 697494-S01
\tFamily: ProLiant

Handle 0x0300, DMI type 3, 21 bytes
chassis information
\tmanufacturer: hp
\ttype: rack mount chassis
\tlock: not present
\tversion: not specified
\tserial number: 2m23360006
\tasset Tag:
\tBoot-up State: Safe
\tPower Supply State: Safe
\tThermal State: Safe
\tSecurity Status: Unknown
\tOEM Information: 0x00000000
\tHeight: 2 U
\tNumber Of Power Cords: 2
\tContained Elements: 0

Handle 0x0401, DMI type 4, 32 bytes
Processor Information
\tSocket Designation: CPU 1
\tType: Central Processor
\tFamily: Other
\tManufacturer: Bochs
\tID: A1 06 02 00 FD FB 8B 17
\tVersion: Not Specified
\tVoltage: Unknown
\tExternal Clock: Unknown
\tMax Speed: 2000 MHz
\tCurrent Speed: 2000 MHz
\tStatus: Populated, Enabled
\tUpgrade: Other
\tL1 Cache Handle: Not Provided
\tL2 Cache Handle: Not Provided
\tL3 Cache Handle: Not Provided

Handle 0x0402, DMI type 4, 32 bytes
Processor Information
\tSocket Designation: CPU 2
\tType: Central Processor
\tFamily: Other
\tManufacturer: Bochs
\tID: A1 06 02 00 FD FB 8B 17
\tVersion: Not Specified
\tVoltage: Unknown
\tExternal Clock: Unknown
\tMax Speed: 2000 MHz
\tCurrent Speed: 2000 MHz
\tStatus: Populated, Enabled
\tUpgrade: Other
\tL1 Cache Handle: Not Provided
\tL2 Cache Handle: Not Provided
\tL3 Cache Handle: Not Provided

Handle 0x0037, DMI type 127, 4 bytes.
End Of Table
'''

DMIDECODE_V = '''
# dmidecode 2.12
SMBIOS 2.4 present.
364 structures occupying 16870 bytes.
Table at 0x000E0010.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: Phoenix Technologies LTD
\tVersion: 6.00
\tRelease Date: 04/14/2014
\tAddress: 0xEA050
\tRuntime Size: 90032 bytes
\tROM Size: 64 kB
\tCharacteristics:
\t\tISA is supported
\t\tPCI is supported
\t\tPC Card (PCMCIA) is supported
\t\tPNP is supported
\t\tAPM is supported
\t\tBIOS is upgradeable
\t\tBIOS shadowing is allowed
\t\tESCD support is available
\t\tBoot from CD is supported
\t\tSelectable boot is supported
\t\tEDD is supported
\t\tPrint screen service is supported (int 5h)
\t\t8042 keyboard services are supported (int 9h)
\t\tSerial services are supported (int 14h)
\t\tPrinter services are supported (int 17h)
\t\tCGA/mono video services are supported (int 10h)
\t\tACPI is supported
\t\tSmart battery is supported
\t\tBIOS boot specification is supported
\t\tFunction key-initiated network boot is supported
\t\tTargeted content distribution is supported
\tBIOS Revision: 4.6
\tFirmware Revision: 0.0

Handle 0x0001, DMI type 1, 27 bytes
System Information
\tManufacturer: VMware, Inc.
\tProduct Name: VMware Virtual Platform
\tVersion: None
\tSerial Number: VMware-42 10 e9 13 1e 11 e2 21-13 c5 c8 1f 11 42 7c cb
\tUUID: 4210E913-1E11-E221-13C5-C81F11427CCB
\tWake-up Type: Power Switch
\tSKU Number: Not Specified
\tFamily: Not Specified
'''

DMIDECODE_AWS = '''
# dmidecode 2.12-dmifs
SMBIOS 2.4 present.
11 structures occupying 310 bytes.
Table at 0x000EB01F.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: Xen
\tVersion: 4.2.amazon
\tRelease Date: 12/09/2016
\tAddress: 0xE8000
\tRuntime Size: 96 kB
\tROM Size: 64 kB
\tCharacteristics:
\t\tPCI is supported
\t\tEDD is supported
\t\tTargeted content distribution is supported
\tBIOS Revision: 4.2

Handle 0x0100, DMI type 1, 27 bytes
System Information
\tManufacturer: Xen
\tProduct Name: HVM domU
\tVersion: 4.2.amazon
\tSerial Number: ec2f58af-2dad-c57e-88c0-a81cb6084290
\tUUID: EC2F58AF-2DAD-C57E-88C0-A81CB6084290
\tWake-up Type: Power Switch
\tSKU Number: Not Specified
\tFamily: Not Specified

Handle 0x0300, DMI type 3, 13 bytes
Chassis Information
\tManufacturer: Xen
\tType: Other
\tLock: Not Present
\tVersion: Not Specified
\tSerial Number: Not Specified
\tAsset Tag: Not Specified
\tBoot-up State: Safe
\tPower Supply State: Safe
\tThermal State: Safe
\tSecurity Status: Unknown
'''

DMIDECODE_KVM = '''
# dmidecode 3.0
Scanning /dev/mem for entry point.
SMBIOS 2.8 present.
13 structures occupying 788 bytes.
Table at 0xBFFFFCE0.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: SeaBIOS
\tVersion: 1.9.1-5.el7_3.2
\tRelease Date: 04/01/2014
\tAddress: 0xE8000
\tRuntime Size: 96 kB
\tROM Size: 64 kB
\tCharacteristics:
\t\tBIOS characteristics not supported
\t\tTargeted content distribution is supported
\tBIOS Revision: 0.0

Handle 0x0100, DMI type 1, 27 bytes
System Information
\tManufacturer: Red Hat
\tProduct Name: RHEV Hypervisor
\tVersion: 7.3-7.el7
\tSerial Number: 34353737-3035-4E43-3734-353130425732
\tUUID: 10331906-BB22-4716-8876-3DFEF8FA941E
\tWake-up Type: Power Switch
\tSKU Number: Not Specified
\tFamily: Red Hat Enterprise Linux

Handle 0x0300, DMI type 3, 21 bytes
Chassis Information
\tManufacturer: Red Hat
\tType: Other
\tLock: Not Present
\tVersion: RHEL 7.2.0 PC (i440FX + PIIX, 1996)
\tSerial Number: Not Specified
\tAsset Tag: Not Specified
\tBoot-up State: Safe
\tPower Supply State: Safe
\tThermal State: Safe
\tSecurity Status: Unknown
\tOEM Information: 0x00000000
\tHeight: Unspecified
\tNumber Of Power Cords: Unspecified
\tContained Elements: 0
'''

DMIDECODE_OVIRT = """
# dmidecode 3.1
Getting SMBIOS data from sysfs.
SMBIOS 2.8 present.
10 structures occupying 511 bytes.
Table at 0x000F6050.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: SeaBIOS
\tVersion: 1.11.0-2.el7
\tRelease Date: 04/01/2014
\tAddress: 0xE8000
\tRuntime Size: 96 kB
\tROM Size: 64 kB
\tCharacteristics:
\t\tBIOS characteristics not supported
\t\tTargeted content distribution is supported
BIOS Revision: 0.0

Handle 0x0100, DMI type 1, 27 bytes
System Information
\tManufacturer: oVirt
\tProduct Name: oVirt Node
\tVersion: 7-5.1804.4.el7.centos
\tSerial Number: 30393137-3436-584D-5136-323830304E46
\tUUID: a35ae32b-ed0a-49a4-9dbb-eecf21f88aab
\tWake-up Type: Power Switch
\tSKU Number: Not Specified
\tFamily: Red Hat Enterprise Linux
"""

DMIDECODE_BOGUS_OVIRT = """
# dmidecode 3.1
Getting SMBIOS data from sysfs.
SMBIOS 2.8 present.
10 structures occupying 511 bytes.
Table at 0x000F6050.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: SeaBIOS
\tVersion: 1.11.0-2.el7
\tRelease Date: 04/01/2014
\tAddress: 0xE8000
\tRuntime Size: 96 kB
\tROM Size: 64 kB
\tCharacteristics:
\t\tBIOS characteristics not supported
\t\tTargeted content distribution is supported
\tBIOS Revision: 0.0

Handle 0x0100, DMI type 1, 27 bytes
System Information
\tManufacturer: oVirt
\tProduct Name: RHEV Hypervisor
\tVersion: 7-5.1804.4.el7.centos
\tSerial Number: 30393137-3436-584D-5136-323830304E46
\tUUID: a35ae32b-ed0a-49a4-9dbb-eecf21f88aab
\tWake-up Type: Power Switch
\tSKU Number: Not Specified
\tFamily: Red Hat Enterprise Linux
"""

DMIDECODE_FAIL = "# dmidecode 2.11\n# No SMBIOS nor DMI entry point found, sorry.\n"


def test_vw_virt_what_1():
    vw = VWP(context_wrap(T1))
    dmi = DMIDecode(context_wrap(DMIDECODE_AWS))
    ret = VirtWhat(dmi, vw)
    assert ret.is_virtual is True
    assert ret.is_physical is False
    assert ret.generic == "kvm"


def test_vw_virt_what_2():
    vw = VWP(context_wrap(T2))
    dmi = DMIDecode(context_wrap(DMIDECODE_AWS))
    ret = VirtWhat(dmi, vw)
    assert ret.is_virtual is False
    assert ret.is_physical is True
    assert ret.generic == "baremetal"


def test_vw_virt_what_specific():
    vw = VWP(context_wrap(T3))
    dmi = DMIDecode(context_wrap(DMIDECODE_AWS))
    ret = VirtWhat(dmi, vw)
    assert ret.is_virtual is True
    assert ret.is_physical is False
    assert ret.generic == "xen"
    assert 'xen-dom0' in ret
    assert 'aws' in ret


def test_vw_dmidecode_1():
    vw = VWP(context_wrap(T4))
    dmi = DMIDecode(context_wrap(DMIDECODE_AWS))
    ret = VirtWhat(dmi, vw)
    assert ret.is_virtual is True
    assert ret.is_physical is False
    assert ret.generic == "xen"
    assert 'aws' in ret


def test_vw_dmidecode_2():
    vw = VWP(context_wrap(T4))
    dmi = DMIDecode(context_wrap(DMIDECODE))
    ret = VirtWhat(dmi, vw)
    assert ret.is_virtual is False
    assert ret.is_physical is True
    assert ret.generic == "baremetal"


def test_vw_dmidecode_3():
    vw = VWP(context_wrap(T4))
    dmi = DMIDecode(context_wrap(DMIDECODE_V))
    ret = VirtWhat(dmi, vw)
    assert ret.is_virtual is True
    assert ret.is_physical is False
    assert ret.generic == "vmware"
    assert ret.amended_generic == "vmware"


def test_vw_dmidecode_4():
    vw = VWP(context_wrap(T4))
    dmi = DMIDecode(context_wrap(DMIDECODE_KVM))
    ret = VirtWhat(dmi, vw)
    assert ret.is_virtual is True
    assert ret.is_physical is False
    assert ret.generic == "kvm"
    assert ret.amended_generic == "rhev"


def test_vw_dmidecode_5():
    dmi = DMIDecode(context_wrap(DMIDECODE_KVM))
    ret = VirtWhat(dmi, None)
    assert ret.is_virtual is True
    assert ret.is_physical is False
    assert ret.generic == "kvm"
    assert ret.amended_generic == "rhev"


def test_vw_dmidecode_is_ovirt():
    vw = VWP(context_wrap(T5))
    dmi = DMIDecode(context_wrap(DMIDECODE_OVIRT))
    ret = VirtWhat(dmi, vw)
    assert ret.is_virtual is True
    assert ret.is_physical is False
    assert ret.generic == "ovirt"
    assert ret.amended_generic == "ovirt"


def test_vw_dmidecode_is_rhev():
    vw = VWP(context_wrap(T5))
    dmi = DMIDecode(context_wrap(DMIDECODE_BOGUS_OVIRT))
    ret = VirtWhat(dmi, vw)
    assert ret.is_virtual is True
    assert ret.is_physical is False
    assert ret.generic == "ovirt"
    assert ret.amended_generic == "rhev"


def test_dmidecode_is_ovirt():
    dmi = DMIDecode(context_wrap(DMIDECODE_OVIRT))
    ret = VirtWhat(dmi, None)
    assert ret.is_virtual is True
    assert ret.is_physical is False
    assert ret.generic == "kvm"
    assert ret.amended_generic == "ovirt"


def test_dmidecode_is_rhev():
    dmi = DMIDecode(context_wrap(DMIDECODE_BOGUS_OVIRT))
    ret = VirtWhat(dmi, None)
    assert ret.is_virtual is True
    assert ret.is_physical is False
    assert ret.generic == "kvm"
    assert ret.amended_generic == "rhev"


def test_virtwhat_only():
    vw = VWP(context_wrap(T5))
    ret = VirtWhat(None, vw)
    assert ret.is_virtual is True
    assert ret.is_physical is False
    assert ret.generic == "ovirt"
    assert ret.amended_generic == "ovirt"



"""
NeutronMetadataAgentLog - file ``/var/log/neutron/metadata-agent.log``
======================================================================
"""

from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.neutron_metadata_agent_log)
class NeutronMetadataAgentLog(LogFileOutput):
    """
      Parse the ``/var/log/neutron/metadata-agent.log`` file.

      .. note::
          Please refer to its super-class :class:`insights.core.LogFileOutput` for more
          details.

      Sample log lines::

          2018-06-08 17:29:55.894 11770 WARNING neutron.agent.metadata.agent [-] Server does not support metadata RPC, fallback to using neutron client
          2018-06-08 17:29:55.907 11770 ERROR neutron.agent.metadata.agent [-] Unexpected error
          2018-06-08 17:29:56.126 11770 TRACE neutron.agent.metadata.agent Traceback (most recent call last):
          2018-06-08 17:29:56.126 11770 TRACE neutron.agent.metadata.agent   File "/usr/lib/python2.7/site-packages/neutron/agent/metadata/agent.py", line 109, in __call__
          2018-06-08 17:29:56.126 11770 TRACE neutron.agent.metadata.agent     self._authenticate_keystone()
          2018-06-08 17:29:56.126 11770 TRACE neutron.agent.metadata.agent   File "/usr/lib/python2.7/site-packages/neutronclient/client.py", line 218, in _authenticate_keystone
          2018-06-08 17:29:56.126 11770 TRACE neutron.agent.metadata.agent     raise exceptions.Unauthorized(message=resp_body)
          2018-06-08 17:29:56.126 11770 TRACE neutron.agent.metadata.agent Unauthorized: {"error": {"message": "The resource could not be found.", "code": 404, "title": "Not Found"}}

      Examples:
          >>> len(metadata_agent_log.get("Server does not support metadata RPC, fallback to using neutron client")) == 1
          True
          >>> from datetime import datetime
          >>> len(list(metadata_agent_log.get_after(datetime(2018, 6, 8, 17, 29, 56))))
          6

      """
    pass



"""
rhsm.conf - File /etc/rhsm/rhsm.conf
====================================

Typical content of "/etc/rhsm/rhsm.conf" is::

    [rhsm]
    # Content base URL:
    baseurl= https://cdn.redhat.com

    # Server CA certificate location:
    ca_cert_dir = /etc/rhsm/ca/

    # Default CA cert to use when generating yum repo configs:
    repo_ca_cert = %(ca_cert_dir)sredhat-uep.pem

    # Where the certificates should be stored
    productCertDir = /etc/pki/product
    entitlementCertDir = /etc/pki/entitlement
    consumerCertDir = /etc/pki/consumer

    # Manage generation of yum repositories for subscribed content:
    manage_repos = 1

    # Refresh repo files with server overrides on every yum command
    full_refresh_on_yum = 0

    # If set to zero, the client will not report the package profile to
    # the subscription management service.
    report_package_profile = 1

    # The directory to search for subscription manager plugins
    pluginDir = /usr/share/rhsm-plugins

    # The directory to search for plugin configuration files
    pluginConfDir = /etc/rhsm/pluginconf.d
"""
from .. import parser, IniConfigFile
from insights.specs import Specs


@parser(Specs.rhsm_conf)
class RHSMConf(IniConfigFile):
    """Parses content of "/etc/rhsm/rhsm.conf". """
    pass



import doctest
import pytest

from insights.parsers import awx_manage, SkipException, ParseException
from insights.core import ContentException
from insights.parsers.awx_manage import AnsibleTowerLicenseType
from insights.tests import context_wrap


NO_LICENSE = """
none
""".strip()

STD_LICENSE = """
enterprise
""".strip()

NG_COMMAND_0 = ""

NG_COMMAND_1 = """
awx-manage: command not found
""".strip()

NG_COMMAND_2 = """
Traceback (most recent call last):
File \"/bin/awx-manage\", line 11, in <module>
    load_entry_point('awx==3.6.4', 'console_scripts', 'awx-manage')()
""".strip()


def test_ansible_tower_license():
    ret = AnsibleTowerLicenseType(context_wrap(NO_LICENSE))
    assert ret.type == 'none'
    ret = AnsibleTowerLicenseType(context_wrap(STD_LICENSE))
    assert ret.type == 'enterprise'


def test_ansible_tower_license_ab():
    with pytest.raises(SkipException):
        AnsibleTowerLicenseType(context_wrap(NG_COMMAND_0))

    with pytest.raises(ContentException):
        AnsibleTowerLicenseType(context_wrap(NG_COMMAND_1))

    with pytest.raises(ParseException):
        AnsibleTowerLicenseType(context_wrap(NG_COMMAND_2))


def test_awx_manage_doc_examples():
    env = {
        'awx_license': AnsibleTowerLicenseType(context_wrap(STD_LICENSE)),
    }
    failed, total = doctest.testmod(awx_manage, globs=env)
    assert failed == 0



"""
CeilometerConf - file ``/etc/ceilometer/ceilometer.conf``
=========================================================

The ``/etc/ceilometer/ceilometer.conf`` file is in a standard '.ini' format,
and this parser uses the IniConfigFile base class to read this.

Given a file containing the following test data::

    [DEFAULT]
    #
    # From ceilometer
    http_timeout = 600
    debug = False
    verbose = False
    log_dir = /var/log/ceilometer
    meter_dispatcher=database
    event_dispatcher=database
    [alarm]
    evaluation_interval = 60
    evaluation_service=ceilometer.alarm.service.SingletonAlarmService
    partition_rpc_topic=alarm_partition_coordination
    [api]
    port = 8777
    host = 192.0.2.10
    [central]
    [collector]
    udp_address = 0.0.0.0
    udp_port = 4952
    [compute]
    [coordination]
    backend_url = redis://:chDWmHdH8dyjsmpCWfCEpJR87@192.0.2.7:6379/

Example:
    >>> config = shared[CeilometerConf]
    >>> config.sections()
    ['DEFAULT', 'alarm', 'api', 'central', 'collector', 'compute', 'coordination']
    >>> config.items('api')
    ['port', 'host']
    >>> config.has_option('alarm', 'evaluation_interval')
    True
    >>> config.get('coordination', 'backend_url')
    'redis://:chDWmHdH8dyjsmpCWfCEpJR87@192.0.2.7:6379/'
    >>> config.getint('collector', 'udp_port')
    4952
    >>> config.getboolean('DEFAULT', 'debug')
    False
"""

from .. import parser, IniConfigFile
from insights.specs import Specs


@parser(Specs.ceilometer_conf)
class CeilometerConf(IniConfigFile):
    """
    A dict of the content of the ``ceilometer.conf`` configuration file.

    Example selection of dictionary contents::

        {
            "DEFAULT": {
                "http_timeout":"600",
                "debug": "False"
             },
            "api": {
                "port":"8877",
            },
        }
    """
    pass



"""VirshListAll - command ``virsh --readonly list --all``
=========================================================

This module provides VM status using output of command ``virsh --readonly list --all``.
"""
from collections import namedtuple

from insights.specs import Specs
from .. import CommandParser, parser
from . import parse_fixed_table, keyword_search


@parser(Specs.virsh_list_all)
class VirshListAll(CommandParser):
    """Parsing output of ``virsh --readonly list --all``.

    Typical output of ``virsh --readonly list --all`` command is::

        Id    Name                           State
        ----------------------------------------------------
        2     rhel7.4                        running
        4     rhel7.0                        paused
        -     centos6.8-router               shut off
        -     cfme-5.7.13                    shut off
        -     cfme-rhos-5.9.0.15             shut off
        -     fedora-24-kernel               shut off
        -     fedora-saio_fedoraSaio         shut off
        -     fedora24-misc                  shut off
        -     freebsd11.0                    shut off
        -     guixSD                         shut off
        -     miq-gap-1                      shut off
        -     rhel7.2                        shut off
        -     RHOSP10                        shut off


    Examples:

        >>> len(output.search(state='shut off')) == 11
        True
        >>> len(output.search(id=None)) == 11
        True
        >>> len(output.search(id=2)) == 1
        True
        >>> output.search(name='rhel7.4') == [{'state': 'running', 'id': 2, 'name': 'rhel7.4'}]
        True
        >>> output.get_vm_state('rhel7.0') == 'paused'
        True
        >>> output.get_vm_state('rhel9.0') is None
        True
        >>> 'cfme' in output
        False
        >>> 'cfme-5.7.13' in output
        True

    Attributes:
        fields (list): List of ``KeyValue`` namedtupules for each line
                       in the command.

        cols (list): List id key value pair derived from the command.

        keywords (list): keywords present in the command, each
                        keyword is converted to lowercase.

    """
    keyvalue = namedtuple('KeyValue',
                          ['name', 'state', 'id', 'name_lower'])
    """namedtuple: Represent name value pair as a namedtuple with case."""
    def _cleanup(self):
        for col in self.cols:
            if col['id'] == '-':
                col['id'] = None
            else:
                col['id'] = (lambda x: int(x) if x.isdigit() else x)(col['id'])

    def parse_content(self, content):
        self.fields = []
        self.cols = []
        self.keywords = []
        if not content:
            return

        self.cols = parse_fixed_table(content,
                                      heading_ignore=['Id', 'Name', 'State'],
                                      header_substitute=[('Id', 'id'), ('Name', 'name'), ('State', 'state')])[1:]  # noqa
        self._cleanup()

        for item in self.cols:
            self.fields.append(self.keyvalue(item['name'], item['state'], item['id'], item['name'].lower()))  # noqa
        self.keywords = [name.name_lower for name in self.fields]

    def __contains__(self, keyword):
        return keyword.lower() in self.keywords

    def __iter__(self):
        return iter(self.fields)

    def search(self, **kw):
        '''Search item based on key value pair.

        Example:

            >>> len(output.search(state='shut off')) == 11
            True
            >>> len(output.search(id=None)) == 11
            True
            >>> len(output.search(id=2)) == 1
            True
        '''
        return keyword_search(self.cols, **kw)

    def get_vm_state(self, vmname):
        '''Get VM state associated with vmname

        Typical output is ``virsh --readonly list --all`` command::

             Id    Name                           State
            ----------------------------------------------------
             2     rhel7.4                        running
             4     rhel7.0                        paused


        Example:

            >>> output.get_vm_state('rhel7.0')
            'paused'

        Args:

            vmname (str): A key. For ex. ``rhel7.0``.

        Returns:

            str: State of VM. Returns None if, ``vmname`` does not exist.
        '''
        vmname = vmname.lower()
        if vmname in self.keywords:
            return self.search(name=vmname)[0]['state']
        return None



import doctest

from insights.tests import context_wrap
from insights.parsers import dnf_conf
from insights.parsers.dnf_conf import DnfConf


DNF_CONF = """
[main]
gpgcheck=1
installonly_limit=3
clean_requirements_on_remove=True
best=False
skip_if_unavailable=True

[rhel-7-server-rhn-tools-beta-debug-rpms]
metadata_expire = 86400
sslclientcert = /etc/pki/entitlement/1234.pem
baseurl = https://cdn.redhat.com/content/beta/rhel/server/7/$basearch/rhn-tools/debug
ui_repoid_vars = basearch
sslverify = 1
name = RHN Tools for Red Hat Enterprise Linux 7 Server Beta (Debug RPMs)
sslclientkey = /etc/pki/entitlement/1234-key.pem
gpgkey = file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-beta,file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release
enabled = 0
sslcacert = /etc/rhsm/ca/redhat-uep.pem
gpgcheck = 1

[bad-repo]
gpgkey =
"""


CONF_PATH = '/etc/dnf/dnf.conf'


def test_doc_examples():
    env = {
        'dconf': DnfConf(context_wrap(DNF_CONF, path=CONF_PATH)),
    }
    failed, total = doctest.testmod(dnf_conf, globs=env)
    assert failed == 0


def test_get_dnf_conf():
    dnf_conf = DnfConf(context_wrap(DNF_CONF, path=CONF_PATH))

    assert dnf_conf.items('main') == {
        'gpgcheck': '1',
        'installonly_limit': '3',
        'clean_requirements_on_remove': 'True',
        'best': 'False',
        'skip_if_unavailable': 'True'
    }

    assert dnf_conf.items('rhel-7-server-rhn-tools-beta-debug-rpms') == {
        u'ui_repoid_vars': u'basearch',
        u'sslverify': u'1',
        u'name': u'RHN Tools for Red Hat Enterprise Linux 7 Server Beta (Debug RPMs)',
        u'sslclientkey': u'/etc/pki/entitlement/1234-key.pem',
        u'enabled': u'0',
        u'gpgkey': [u'file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-beta',
                    u'file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release'],
        u'sslclientcert': u'/etc/pki/entitlement/1234.pem',
        u'baseurl': [u'https://cdn.redhat.com/content/beta/rhel/server/7/$basearch/rhn-tools/debug'],
        u'sslcacert': u'/etc/rhsm/ca/redhat-uep.pem',
        u'gpgcheck': u'1',
        u'metadata_expire': u'86400'
    }

    assert dnf_conf.file_name == 'dnf.conf'
    assert dnf_conf.file_path == '/etc/dnf/dnf.conf'



import os

__all__ = ("verify", "PlaybookValidationError")


class PlaybookValidationError(Exception):
    """
    Exception raised when playbook validation fails

    Attributes:
        playbook -- stringified playbook yaml from stdin
        message -- explanation of why validation failed
    """

    def __init__(self, message="PLAYBOOK VALIDATION FAILED"):
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return self.message


def verify(unverified_playbook):
    """
    Verify the signed playbook.

    Input: stringified "broken" unverified playbook
    Output: stringified "verified" playbook
    Error: exception
    """
    # Skeleton implementation ... "bless" the incoming playbook
    ERROR = os.getenv('ANSIBLE_PLAYBOOK_VERIFIER_THROW_ERROR')
    if ERROR:
        raise PlaybookValidationError()

    verified_playbook = unverified_playbook
    return verified_playbook



"""
NVMeCoreIOTimeout - The timeout for I/O operations submitted to NVMe devices
============================================================================

This parser reads the content of ``/sys/module/nvme_core/parameters/io_timeout``.
"""

from insights import Parser, parser
from insights.specs import Specs
from ..parsers import SkipException, ParseException


@parser(Specs.nvme_core_io_timeout)
class NVMeCoreIOTimeout(Parser):
    """
    Class for parsing the content of the ``/sys/module/nvme_core/parameters/io_timeout``.

    A typical sample of the content of this file looks like::

        4294967295

    Raises:
        SkipException: When content is empty or no parse-able content.
        ParseException: When type cannot be recognized.

    Attributes:
        val (int): It is used to show the current value of the timeout for I/O operations submitted to NVMe devices.

    Examples:
        >>> nciotmo.val
        4294967295
    """

    def parse_content(self, content):
        if not content or len(content) != 1:
            raise SkipException()
        if not content[0].strip('').isdigit():
            raise ParseException("Unexpected content: ", content)
        self.val = int(content[0].strip())



from insights.parsers import openshift_get
from insights.tests import context_wrap
import datetime
import doctest

OC_GET_POD = """
apiVersion: v1
items:
- apiVersion: v1
  kind: Pod
  metadata:
    annotations:
      openshift.io/scc: anyuid
    creationTimestamp: 2017-02-10T16:33:46Z
    labels:
      name: hello-openshift
    name: hello-pod
    namespace: default
  spec:
    containers:
    - image: openshift/hello-openshift
      imagePullPolicy: IfNotPresent
      name: hello-openshift
      ports:
      - containerPort: 8080
        protocol: TCP
      resources: {}
      securityContext:
        capabilities:
          drop:
          - MKNOD
          - SYS_CHROOT
        privileged: false
        seLinuxOptions:
          level: s0:c5,c0
      terminationMessagePath: /dev/termination-log
      volumeMounts:
      - mountPath: /var/run/secrets/kubernetes.io/serviceaccount
        name: default-token-yk69f
        readOnly: true
    dnsPolicy: ClusterFirst
    host: node2.ose.com
    imagePullSecrets:
    - name: default-dockercfg-h7sl1
    nodeName: node2.ose.com
    restartPolicy: Always
    securityContext:
      seLinuxOptions:
        level: s0:c5,c0
    serviceAccount: default
    serviceAccountName: default
    terminationGracePeriodSeconds: 30
    volumes:
    - name: default-token-yk69f
      secret:
        secretName: default-token-yk69f
  status:
    conditions:
    - lastProbeTime: null
      lastTransitionTime: 2017-02-10T16:33:46Z
      status: "True"
      type: Initialized
    containerStatuses:
    - containerID: docker://a172a6945e207c0fd1c391cad31ccb76bc8323f3f024a50b5ba287034302853f
      image: openshift/hello-openshift
      imageID: docker-pullable://docker.io/openshift/hello-openshift@sha256:9b1b29dc4ed029220b2d87fce57fab43f450fa6521ab86f22ddbc5ecc978752a
      lastState:
        terminated:
          containerID: docker://9ea5647da89302437630e96728f9f65593c07d0e65a1b275854fcb4c738c8c46
          exitCode: 2
          finishedAt: 2017-02-13T18:59:47Z
          reason: Error
          startedAt: 2017-02-10T16:33:56Z
      name: hello-openshift
      ready: true
      restartCount: 1
      state:
        running:
          startedAt: 2017-02-13T19:00:49Z
    hostIP: 10.66.208.105
    phase: Running
    podIP: 10.1.0.3
    startTime: 2017-02-10T16:33:46Z
- apiVersion: v1
  kind: Pod
  metadata:
    annotations:
      kubernetes.io/created-by: |
        {"kind":"SerializedReference","apiVersion":"v1","reference":{"kind":"ReplicationController","namespace":"zjj-project","name":"router-1-1","uid":"12c1a374-f75a-11e6-80d0-001a4a0100d2","apiVersion":"v1","resourceVersion":"1638409"}}
      openshift.io/deployment-config.latest-version: "1"
      openshift.io/deployment-config.name: router-1
      openshift.io/deployment.name: router-1-1
      openshift.io/scc: hostnetwork
    creationTimestamp: 2017-02-20T10:48:14Z
    generateName: router-1-1-
    labels:
      deployment: router-1-1
      deploymentconfig: router-1
      router: router-1
    name: router-1-1-w27o2
  spec:
    containers:
    - env:
      - name: DEFAULT_CERTIFICATE_DIR
        value: /etc/pki/tls/private
      - name: ROUTER_EXTERNAL_HOST_HOSTNAME
      - name: ROUTER_EXTERNAL_HOST_HTTPS_VSERVER
      - name: ROUTER_EXTERNAL_HOST_HTTP_VSERVER
      - name: ROUTER_EXTERNAL_HOST_INSECURE
        value: "false"
      - name: ROUTER_EXTERNAL_HOST_PARTITION_PATH
      - name: ROUTER_EXTERNAL_HOST_PASSWORD
      - name: ROUTER_EXTERNAL_HOST_PRIVKEY
        value: /etc/secret-volume/router.pem
      - name: ROUTER_EXTERNAL_HOST_USERNAME
      - name: ROUTER_SERVICE_HTTPS_PORT
        value: "443"
      - name: ROUTER_SERVICE_HTTP_PORT
        value: "80"
      - name: ROUTER_SERVICE_NAME
        value: router-1
      - name: ROUTER_SERVICE_NAMESPACE
        value: zjj-project
      - name: ROUTER_SUBDOMAIN
      - name: STATS_PASSWORD
        value: password
      - name: STATS_PORT
        value: "1936"
      - name: STATS_USERNAME
        value: admin
      image: openshift3/ose-haproxy-router:v3.3.1.7
      imagePullPolicy: IfNotPresent
      livenessProbe:
        failureThreshold: 3
        httpGet:
          host: localhost
          path: /healthz
          port: 1936
          scheme: HTTP
        initialDelaySeconds: 10
        periodSeconds: 10
        successThreshold: 1
        timeoutSeconds: 1
      name: router
      ports:
      - containerPort: 80
        hostPort: 80
        protocol: TCP
      - containerPort: 443
        hostPort: 443
        protocol: TCP
      - containerPort: 1936
        hostPort: 1936
        name: stats
        protocol: TCP
      readinessProbe:
        failureThreshold: 3
        httpGet:
          host: localhost
          path: /healthz
          port: 1936
          scheme: HTTP
        initialDelaySeconds: 10
        periodSeconds: 10
        successThreshold: 1
        timeoutSeconds: 1
      resources:
        requests:
          cpu: 100m
          memory: 256Mi
      securityContext:
        capabilities:
          drop:
          - KILL
          - MKNOD
          - SETGID
          - SETUID
          - SYS_CHROOT
        privileged: false
        runAsUser: 1000070000
        seLinuxOptions:
          level: s0:c8,c7
      terminationMessagePath: /dev/termination-log
      volumeMounts:
      - mountPath: /etc/pki/tls/private
        name: server-certificate
        readOnly: true
      - mountPath: /var/run/secrets/kubernetes.io/serviceaccount
        name: router-token-0j7an
        readOnly: true
    dnsPolicy: ClusterFirst
    host: node1.ose.com
    hostNetwork: true
    imagePullSecrets:
    - name: router-dockercfg-dlu6n
    nodeName: node1.ose.com
    restartPolicy: Always
    securityContext:
      fsGroup: 1000070000
      seLinuxOptions:
        level: s0:c8,c7
      supplementalGroups:
      - 1000070000
    serviceAccount: router
    serviceAccountName: router
    terminationGracePeriodSeconds: 30
    volumes:
    - name: server-certificate
      secret:
        secretName: router-1-certs
    - name: router-token-0j7an
      secret:
        secretName: router-token-0j7an
  status:
    conditions:
    - lastProbeTime: null
      lastTransitionTime: 2017-02-20T10:48:14Z
      status: "True"
      type: Initialized
    containerStatuses:
    - containerID: docker://aa4348a647e0f3186e70a0ce9837f84a25060b4daebab370c1fc093cf8af3349
      image: openshift3/ose-haproxy-router:v3.3.1.7
      imageID: docker-pullable://registry.access.redhat.com/openshift3/ose-haproxy-router@sha256:f2f75cfd2b828c3143ca8022e26593a7491ca040dab6d6472472ed040d1c1b83
      lastState: {}
      name: router
      ready: true
      restartCount: 0
      state:
        running:
          startedAt: 2017-02-20T10:48:16Z
    hostIP: 10.66.208.229
    phase: Running
    podIP: 10.66.208.229
    startTime: 2017-02-20T10:48:14Z
kind: List
metadata: {}
""".strip()

OC_GET_SERVICE = """
apiVersion: v1
items:
- apiVersion: v1
  kind: Service
  metadata:
    creationTimestamp: 2016-12-27T03:24:03Z
    labels:
      component: apiserver
      provider: kubernetes
    name: kubernetes
    namespace: default
    resourceVersion: "9"
    selfLink: /api/v1/namespaces/default/services/kubernetes
    uid: ea9d8fb4-cbe3-11e6-b3c1-001a4a0100d2
  spec:
    clusterIP: 172.30.0.1
    portalIP: 172.30.0.1
    ports:
    - name: https
      port: 443
      protocol: TCP
      targetPort: 443
    - name: dns
      port: 53
      protocol: UDP
      targetPort: 8053
    - name: dns-tcp
      port: 53
      protocol: TCP
      targetPort: 8053
    sessionAffinity: ClientIP
    type: ClusterIP
  status:
    loadBalancer: {}
- apiVersion: v1
  kind: Service
  metadata:
    annotations:
      service.alpha.openshift.io/serving-cert-secret-name: router-1-certs
      service.alpha.openshift.io/serving-cert-signed-by: openshift-service-serving-signer@1480042702
    creationTimestamp: 2017-02-20T10:48:11Z
    labels:
      router: router-1
    name: router-1
    namespace: zjj-project
    resourceVersion: "1638401"
    selfLink: /api/v1/namespaces/zjj-project/services/router-1
    uid: 12bdf634-f75a-11e6-80d0-001a4a0100d2
  spec:
    clusterIP: 172.30.210.0
    portalIP: 172.30.210.0
    ports:
    - name: 80-tcp
      port: 80
      protocol: TCP
      targetPort: 80
    - name: 443-tcp
      port: 443
      protocol: TCP
      targetPort: 443
    - name: 1936-tcp
      port: 1936
      protocol: TCP
      targetPort: 1936
    selector:
      router: router-1
    sessionAffinity: None
    type: ClusterIP
  status:
    loadBalancer: {}
kind: List
metadata: {}
""".strip()

OC_GET_CONFIGMAP = """
apiVersion: v1
items:
- apiVersion: v1
  data:
    node-config.yaml: |
      apiVersion: v1
      authConfig:
        authenticationCacheSize: 1000
        authenticationCacheTTL: 5m
        authorizationCacheSize: 1000
        authorizationCacheTTL: 5m
      dnsBindAddress: 127.0.0.1:53
      dnsDomain: cluster.local
      dnsIP: 0.0.0.0
      dnsNameservers: null
      dnsRecursiveResolvConf: /etc/origin/node/resolv.conf
      dockerConfig:
        dockerShimRootDirectory: /var/lib/dockershim
        dockerShimSocket: /var/run/dockershim.sock
        execHandlerName: native
      enableUnidling: true
      imageConfig:
        format: registry.access.redhat.com/openshift3/ose-${component}:${version}
        latest: false
      iptablesSyncPeriod: 30s
      kind: NodeConfig
      kubeletArguments:
        bootstrap-kubeconfig:
        - /etc/origin/node/bootstrap.kubeconfig
        cert-dir:
        - /etc/origin/node/certificates
        cloud-config:
        - /etc/origin/cloudprovider/openstack.conf
        cloud-provider:
        - openstack
        container-runtime:
        - remote
        container-runtime-endpoint:
        - /var/run/crio/crio.sock
        enable-controller-attach-detach:
        - 'true'
        feature-gates:
        - RotateKubeletClientCertificate=true,RotateKubeletServerCertificate=true
        image-service-endpoint:
        - /var/run/crio/crio.sock
        node-labels:
        - node-role.kubernetes.io/infra=true,node-role.kubernetes.io/master=true,node-role.kubernetes.io/compute=true
        pod-manifest-path:
        - /etc/origin/node/pods
        rotate-certificates:
        - 'true'
        runtime-request-timeout:
        - 10m
      masterClientConnectionOverrides:
        acceptContentTypes: application/vnd.kubernetes.protobuf,application/json
        burst: 40
        contentType: application/vnd.kubernetes.protobuf
        qps: 20
      masterKubeConfig: node.kubeconfig
      networkConfig:
        mtu: 1450
        networkPluginName: redhat/openshift-ovs-networkpolicy
      servingInfo:
        bindAddress: 0.0.0.0:10250
        bindNetwork: tcp4
        clientCA: client-ca.crt
      volumeConfig:
        localQuota:
          perFSGroup: null
      volumeDirectory: /var/lib/origin/openshift.local.volumes
    volume-config.yaml: |
      apiVersion: kubelet.config.openshift.io/v1
      kind: VolumeConfig
      localQuota:
        perFSGroup: 512Mi
  kind: ConfigMap
  metadata:
    creationTimestamp: 2018-08-07T14:56:15Z
    name: node-config-all-in-one
    namespace: openshift-node
    resourceVersion: "1057"
    selfLink: /api/v1/namespaces/openshift-node/configmaps/node-config-all-in-one
    uid: 0856aa7e-9a52-11e8-b4f0-fa163e0d4482
- apiVersion: v1
  data:
    fluent.conf: |
      # This file is the fluentd configuration entrypoint. Edit with care.

      @include configs.d/openshift/system.conf

      # In each section below, pre- and post- includes don't include anything initially;
      # they exist to enable future additions to openshift conf as needed.

      ## sources
      ## ordered so that syslog always runs last...
      @include configs.d/openshift/input-pre-*.conf
      @include configs.d/dynamic/input-docker-*.conf
      @include configs.d/dynamic/input-syslog-*.conf
      @include configs.d/openshift/input-post-*.conf
      ##

      <label @INGRESS>
      ## filters
        @include configs.d/openshift/filter-pre-*.conf
        @include configs.d/openshift/filter-retag-journal.conf
        @include configs.d/openshift/filter-k8s-meta.conf
        @include configs.d/openshift/filter-kibana-transform.conf
        @include configs.d/openshift/filter-k8s-flatten-hash.conf
        @include configs.d/openshift/filter-k8s-record-transform.conf
        @include configs.d/openshift/filter-syslog-record-transform.conf
        @include configs.d/openshift/filter-viaq-data-model.conf
        @include configs.d/openshift/filter-post-*.conf
      ##
      </label>

      <label @OUTPUT>
      ## matches
        @include configs.d/openshift/output-pre-*.conf
        @include configs.d/openshift/output-operations.conf
        @include configs.d/openshift/output-applications.conf
        # no post - applications.conf matches everything left
      ##
      </label>
    secure-forward.conf: |
      # <store>
      # @type secure_forward

      # self_hostname ${hostname}
      # shared_key <SECRET_STRING>

      # secure yes
      # enable_strict_verification yes

      # ca_cert_path /etc/fluent/keys/your_ca_cert
      # ca_private_key_path /etc/fluent/keys/your_private_key
        # for private CA secret key
      # ca_private_key_passphrase passphrase

      # <server>
        # or IP
      #   host server.fqdn.example.com
      #   port 24284
      # </server>
      # <server>
        # ip address to connect
      #   host 203.0.113.8
        # specify hostlabel for FQDN verification if ipaddress is used for host
      #   hostlabel server.fqdn.example.com
      # </server>
      # </store>
    throttle-config.yaml: |
      # Logging example fluentd throttling config file

      #example-project:
      #  read_lines_limit: 10
      #
      #.operations:
      #  read_lines_limit: 100
  kind: ConfigMap
  metadata:
    creationTimestamp: 2018-08-07T15:36:51Z
    name: logging-fluentd
    namespace: openshift-logging
    resourceVersion: "8922"
    selfLink: /api/v1/namespaces/openshift-logging/configmaps/logging-fluentd
    uid: b4b8f35c-9a57-11e8-b5ba-fa163e0d4482
kind: List
metadata:
  resourceVersion: ""
  selfLink: ""
""".strip()

OC_GET_BC = """
apiVersion: v1
items:
- apiVersion: v1
  kind: BuildConfig
  metadata:
    annotations:
      openshift.io/generated-by: OpenShiftWebConsole
    creationTimestamp: 2017-11-28T09:02:19Z
    labels:
      app: tom
    name: tom
    namespace: ci
    resourceVersion: "8922062"
    selfLink: /oapi/v1/namespaces/ci/buildconfigs/tom
    uid: d6a0364c-d41a-11e7-aef1-001a4a010222
  spec:
    nodeSelector: null
    output:
      to:
        kind: ImageStreamTag
        name: tom:latest
    postCommit: {}
    resources:
      limits:
        cpu: "1"
        memory: 512m
      requests:
        cpu: 100m
        memory: 256m
    runPolicy: Serial
    source:
      contextDir: tomcat-websocket-chat
      git:
        ref: master
        uri: https://github.com/jboss-openshift/openshift-quickstarts.git
      type: Git
    strategy:
      sourceStrategy:
        from:
          kind: ImageStreamTag
          name: jboss-webserver30-tomcat7-openshift:1.3
          namespace: openshift
      type: Source
    triggers:
    - generic:
        secret: 222545c18370f300
      type: Generic
    - github:
        secret: a75ac7b89f8afc22
      type: GitHub
    - imageChange:
        lastTriggeredImageID: registry.access.redhat.com/jboss-webserver-3/webserver30-tomcat7-openshift@sha256:6ee8de68a744d820e249f784b5ba41d059f91a5554fce47c7e0b998aa88c97cb
      type: ImageChange
    - type: ConfigChange
  status:
    lastVersion: 2
- apiVersion: v1
  kind: BuildConfig
  metadata:
    annotations:
      openshift.io/generated-by: OpenShiftWebConsole
    creationTimestamp: 2017-11-28T09:05:26Z
    labels:
      app: mybank
    name: mybank
    namespace: mybank
    resourceVersion: "8961033"
    selfLink: /oapi/v1/namespaces/mybank/buildconfigs/mybank
    uid: 46711668-d41b-11e7-aef1-001a4a010222
  spec:
    nodeSelector: null
    output:
      to:
        kind: ImageStreamTag
        name: mybank:latest
    postCommit: {}
    resources:
      limits:
        cpu: "1"
        memory: 512m
      requests:
        cpu: 100m
        memory: 256M
    runPolicy: Serial
    source:
      git:
        ref: master
        uri: https://github.com/shzhou12/mybank-demo-maven
      type: Git
    strategy:
      sourceStrategy:
        from:
          kind: ImageStreamTag
          name: jboss-eap70-openshift:1.5
          namespace: openshift
      type: Source
    triggers:
    - generic:
        secret: 3bf9b0f9eda5bcc3
      type: Generic
    - github:
        secret: a1cea7fe7310d6df
      type: GitHub
    - imageChange:
        lastTriggeredImageID: registry.access.redhat.com/jboss-eap-7/eap70-openshift@sha256:b1b664a9b6f797d530bf12c29123947c1feb4590336ecc9d118b3f2e44000524
      type: ImageChange
    - type: ConfigChange
  status:
    lastVersion: 11
kind: List
metadata: {}
resourceVersion: ""
selfLink: ""
""".strip()

OC_GET_DC = """
apiVersion: v1
items:
- apiVersion: v1
  kind: DeploymentConfig
  metadata:
    creationTimestamp: 2017-02-14T15:21:51Z
    generation: 3
    labels:
      docker-registry: default
    name: docker-registry
    namespace: openshift
    resourceVersion: "1439616"
    selfLink: /oapi/v1/namespaces/openshift/deploymentconfigs/docker-registry
    uid: 4f1cf726-f2c9-11e6-8c0e-001a4a0100d2
  spec:
    replicas: 1
    selector:
      docker-registry: default
    strategy:
      resources: {}
      rollingParams:
        intervalSeconds: 1
        maxSurge: 25%
        maxUnavailable: 25%
        timeoutSeconds: 600
        updatePeriodSeconds: 1
      type: Rolling
    template:
      metadata:
        creationTimestamp: null
        labels:
          docker-registry: default
      spec:
        containers:
        - env:
          - name: REGISTRY_HTTP_ADDR
            value: :5000
          - name: REGISTRY_HTTP_NET
            value: tcp
          - name: REGISTRY_HTTP_SECRET
            value: mysupersecrethttpsecret
          - name: REGISTRY_MIDDLEWARE_REPOSITORY_OPENSHIFT_ENFORCEQUOTA
            value: "false"
          image: registry.access.redhat.com/openshift3/ose-docker-registry
          imagePullPolicy: Always
          livenessProbe:
            failureThreshold: 3
            httpGet:
              path: /healthz
              port: 5000
              scheme: HTTP
            initialDelaySeconds: 10
            periodSeconds: 10
            successThreshold: 1
            timeoutSeconds: 5
          name: registry
          ports:
          - containerPort: 5000
            protocol: TCP
          readinessProbe:
            failureThreshold: 3
            httpGet:
              path: /healthz
              port: 5000
              scheme: HTTP
            periodSeconds: 10
            successThreshold: 1
            timeoutSeconds: 5
          resources:
            requests:
              cpu: 100m
              memory: 256Mi
          securityContext:
            privileged: false
          terminationMessagePath: /dev/termination-log
          volumeMounts:
          - mountPath: /registry
            name: registry-storage
        dnsPolicy: ClusterFirst
        terminationGracePeriodSeconds: 30
        volumes:
        - name: registry-storage
          persistentVolumeClaim:
            claimName: registry-claim-test1
    test: false
    triggers:
    - type: ConfigChange
  status:
    availableReplicas: 1
    details:
      causes:
      - type: ConfigChange
      message: caused by a config change
    latestVersion: 3
    observedGeneration: 3
    replicas: 1
    updatedReplicas: 1
- apiVersion: v1
  kind: DeploymentConfig
  metadata:
    creationTimestamp: 2017-02-20T10:48:11Z
    generation: 1
    labels:
      router: router-1
    name: router-1
    namespace: zjj-project
    resourceVersion: "1638435"
    selfLink: /oapi/v1/namespaces/zjj-project/deploymentconfigs/router-1
    uid: 12b9b90f-f75a-11e6-80d0-001a4a0100d2
  spec:
    replicas: 1
    selector:
      router: router-1
    strategy:
      resources: {}
      rollingParams:
        intervalSeconds: 1
        maxSurge: 0
        maxUnavailable: 25%
        timeoutSeconds: 600
        updatePercent: -25
        updatePeriodSeconds: 1
      type: Rolling
    template:
      metadata:
        creationTimestamp: null
        labels:
          router: router-1
      spec:
        containers:
        - env:
          - name: DEFAULT_CERTIFICATE_DIR
            value: /etc/pki/tls/private
          - name: ROUTER_EXTERNAL_HOST_HOSTNAME
          - name: ROUTER_EXTERNAL_HOST_HTTPS_VSERVER
          - name: ROUTER_EXTERNAL_HOST_HTTP_VSERVER
          - name: ROUTER_EXTERNAL_HOST_INSECURE
            value: "false"
          - name: ROUTER_EXTERNAL_HOST_PARTITION_PATH
          - name: ROUTER_EXTERNAL_HOST_PASSWORD
          - name: ROUTER_EXTERNAL_HOST_PRIVKEY
            value: /etc/secret-volume/router.pem
          - name: ROUTER_EXTERNAL_HOST_USERNAME
          - name: ROUTER_SERVICE_HTTPS_PORT
            value: "443"
          - name: ROUTER_SERVICE_HTTP_PORT
            value: "80"
          - name: ROUTER_SERVICE_NAME
            value: router-1
          - name: ROUTER_SERVICE_NAMESPACE
            value: zjj-project
          - name: ROUTER_SUBDOMAIN
          - name: STATS_PASSWORD
            value: password
          - name: STATS_PORT
            value: "1936"
          - name: STATS_USERNAME
            value: admin
          image: openshift3/ose-haproxy-router:v3.3.1.7
          imagePullPolicy: IfNotPresent
          livenessProbe:
            failureThreshold: 3
            httpGet:
              host: localhost
              path: /healthz
              port: 1936
              scheme: HTTP
            initialDelaySeconds: 10
            periodSeconds: 10
            successThreshold: 1
            timeoutSeconds: 1
          name: router
          ports:
          - containerPort: 1936
            hostPort: 1936
            name: stats
            protocol: TCP
          readinessProbe:
            failureThreshold: 3
            httpGet:
              host: localhost
              path: /healthz
              port: 1936
              scheme: HTTP
            initialDelaySeconds: 10
            periodSeconds: 10
            successThreshold: 1
            timeoutSeconds: 1
          resources:
            requests:
              cpu: 100m
              memory: 256Mi
          terminationMessagePath: /dev/termination-log
          volumeMounts:
          - mountPath: /etc/pki/tls/private
            name: server-certificate
            readOnly: true
        dnsPolicy: ClusterFirst
        volumes:
        - name: server-certificate
          secret:
            secretName: router-1-certs
    test: false
    triggers:
    - type: ConfigChange
  status:
    availableReplicas: 1
    details:
      causes:
      - type: ConfigChange
      message: caused by a config change
    latestVersion: 1
    observedGeneration: 1
    replicas: 1
    updatedReplicas: 1
kind: List
metadata: {}
""".strip()

OC_GET_ROLEBINDING = """
apiVersion: v1
items:
- apiVersion: v1
  groupNames: null
  kind: RoleBinding
  metadata:
    creationTimestamp: 2017-03-07T09:00:56Z
    name: admin
    namespace: foo
    resourceVersion: "11803596"
    selfLink: /oapi/v1/namespaces/foo/rolebindings/admin
    uid: 93256034-0314-11e7-b98e-001a4a0101f0
  roleRef:
    name: admin
  subjects:
  - kind: SystemUser
    name: system:admin
  userNames:
  - system:admin
- apiVersion: v1
  groupNames: null
  kind: RoleBinding
  metadata:
    creationTimestamp: 2017-03-07T09:00:56Z
    name: system:image-builders
    namespace: foo
    resourceVersion: "11803603"
    selfLink: /oapi/v1/namespaces/foo/rolebindings/system:image-builders
    uid: 93709567-0314-11e7-b98e-001a4a0101f0
  roleRef:
    name: system:image-builder
  subjects:
  - kind: ServiceAccount
    name: builder
    namespace: foo
  userNames:
  - system:serviceaccount:foo:builder
- apiVersion: v1
  groupNames: null
  kind: RoleBinding
  metadata:
    creationTimestamp: null
    name: myrole
    namespace: foo
    resourceVersion: "415"
    selfLink: /oapi/v1/namespaces/foo/rolebindings/myrole
  roleRef:
    name: myrole
    namespace: foo
  subjects: null
  userNames: null
kind: List
metadata: {}
""".strip()

OC_GET_PROJECT = """
apiVersion: v1
items:
- apiVersion: v1
  kind: Project
  metadata:
    annotations:
      openshift.io/description: ""
      openshift.io/display-name: ""
      openshift.io/requester: testuser
      openshift.io/sa.scc.mcs: s0:c8,c2
      openshift.io/sa.scc.supplemental-groups: 1000060000/10000
      openshift.io/sa.scc.uid-range: 1000060000/10000
    creationTimestamp: 2017-02-13T03:01:30Z
    name: zjj-project
    resourceVersion: "11040756"
    selfLink: /oapi/v1/projects/zjj-project
    uid: b83cdc59-f198-11e6-b98e-001a4a0101f0
  spec:
    finalizers:
    - openshift.io/origin
    - kubernetes
  status:
    phase: Active
- apiVersion: v1
  kind: Project
  metadata:
    annotations:
      openshift.io/description: ""
      openshift.io/display-name: ""
      openshift.io/requester: testuser
      openshift.io/sa.scc.mcs: s0:c11,c0
      openshift.io/sa.scc.supplemental-groups: 1000110000/10000
      openshift.io/sa.scc.uid-range: 1000110000/10000
    creationTimestamp: 2016-12-27T07:49:13Z
    name: test
    resourceVersion: "9401953"
    selfLink: /oapi/v1/projects/test
    uid: f5f2a52c-cc08-11e6-8b9b-001a4a0101f0
  spec:
    finalizers:
    - openshift.io/origin
    - kubernetes
  status:
    phase: Active
kind: List
metadata: {}
""".strip()

OC_GET_ROLE = """
apiVersion: v1
items:
- apiVersion: v1
  kind: Role
  metadata:
    creationTimestamp: 2016-08-30T16:13:03Z
    name: shared-resource-viewer
    namespace: openshift
    resourceVersion: "94"
    selfLink: /oapi/v1/namespaces/openshift/roles/shared-resource-viewer
    uid: a10c3f88-6ecc-11e6-83c6-001a4a0101f0
  rules:
  - apiGroups: null
    attributeRestrictions: null
    resources:
    - imagestreamimages
    - imagestreamimports
    - imagestreammappings
    - imagestreams
    - imagestreamtags
    - templates
    verbs:
    - get
    - list
  - apiGroups: null
    attributeRestrictions: null
    resources:
    - imagestreams/layers
    verbs:
    - get
kind: List
metadata: {}
""".strip()


OC_GET_ROUTE = """
apiVersion: v1
items:
- apiVersion: v1
  kind: Route
  metadata:
    creationTimestamp: 2018-02-23T04:26:01Z
    name: docker-registry
    namespace: default
    resourceVersion: "4166"
    selfLink: /oapi/v1/namespaces/default/routes/docker-registry
    uid: a72a9680-1851-11e8-82e4-001a4a0102eb
  spec:
    host: docker-registry-default.router.default.svc.cluster.local
    tls:
      termination: passthrough
    to:
      kind: Service
      name: docker-registry
      weight: 100
    wildcardPolicy: None
  status:
    ingress:
    - conditions:
      - lastTransitionTime: 2018-02-23T06:24:02Z
        status: "True"
        type: Admitted
      host: docker-registry-default.router.default.svc.cluster.local
      routerName: router
      wildcardPolicy: None
kind: List
metadata: {}
resourceVersion: ""
selfLink: ""
""".strip()


OC_GET_PV = """
apiVersion: v1
items:
- apiVersion: v1
  kind: PersistentVolume
  metadata:
    annotations:
      pv.kubernetes.io/bound-by-controller: "yes"
    creationTimestamp: 2017-03-09T15:23:17Z
    name: registry-volume
    resourceVersion: "745"
    selfLink: /api/v1/persistentvolumes/registry-volume
    uid: 52394d79-04dc-11e7-ab1f-001a4a0101d2
  spec:
    accessModes:
    - ReadWriteMany
    capacity:
      storage: 5Gi
    claimRef:
      apiVersion: v1
      kind: PersistentVolumeClaim
      name: registry-claim
      namespace: default
      resourceVersion: "743"
      uid: 52c44068-04dc-11e7-ab1f-001a4a0101d2
    nfs:
      path: /exports/registry
      server: master.ose33.com
    persistentVolumeReclaimPolicy: Retain
  status:
    phase: Bound
- apiVersion: v1
  kind: PersistentVolume
  metadata:
    creationTimestamp: 2017-04-05T10:44:54Z
    name: registry-volume-zjj
    resourceVersion: "934892"
    selfLink: /api/v1/persistentvolumes/registry-volume-zjj
    uid: e7519f6c-19ec-11e7-ab1f-001a4a0101d2
  spec:
    accessModes:
    - ReadWriteMany
    capacity:
      storage: 10Gi
    nfs:
      path: /nfs
      server: 10.66.208.147
    persistentVolumeReclaimPolicy: Recycle
  status:
    phase: Available
kind: List
metadata: {}
""".strip()

OC_GET_PVC = """
apiVersion: v1
items:
- apiVersion: v1
  kind: PersistentVolumeClaim
  metadata:
    annotations:
      pv.kubernetes.io/bind-completed: "yes"
      pv.kubernetes.io/bound-by-controller: "yes"
    creationTimestamp: 2017-03-09T15:23:18Z
    name: registry-claim
    namespace: default
    resourceVersion: "747"
    selfLink: /api/v1/namespaces/default/persistentvolumeclaims/registry-claim
    uid: 52c44068-04dc-11e7-ab1f-001a4a0101d2
  spec:
    accessModes:
    - ReadWriteMany
    resources:
      requests:
        storage: 5Gi
    volumeName: registry-volume
  status:
    accessModes:
    - ReadWriteMany
    capacity:
      storage: 5Gi
    phase: Bound
- apiVersion: v1
  kind: PersistentVolumeClaim
  metadata:
    annotations:
      pv.kubernetes.io/bind-completed: "yes"
    creationTimestamp: 2017-04-12T18:40:43Z
    name: registry-claim-test1
    namespace: default
    resourceVersion: "1084833"
    selfLink: /api/v1/namespaces/default/persistentvolumeclaims/registry-claim-test1
    uid: 89169428-1faf-11e7-b236-001a4a0101d2
  spec:
    accessModes:
    - ReadWriteMany
    resources:
      requests:
        storage: 5Gi
    volumeName: registry-volume-zjj
  status:
    accessModes:
    - ReadWriteMany
    capacity:
      storage: 10Gi
    phase: Bound
kind: List
metadata: {}
""".strip()

OC_GET_ENDPOINTS = """
apiVersion: v1
items:
- apiVersion: v1
  kind: Endpoints
  metadata:
    creationTimestamp: 2017-06-15T05:53:47Z
    name: gluster-cluster
    namespace: default
    resourceVersion: "35151"
    selfLink: /api/v1/namespaces/default/endpoints/gluster-cluster
    uid: ffaf2c59-518e-11e7-a93b-001a4a01010c
  subsets:
  - addresses:
    - ip: 10.64.221.124
    - ip: 10.64.221.126
    ports:
    - port: 1
      protocol: TCP
- apiVersion: v1
  kind: Endpoints
  metadata:
    creationTimestamp: 2017-06-14T05:55:59Z
    name: kubernetes
    namespace: default
    resourceVersion: "449"
    selfLink: /api/v1/namespaces/default/endpoints/kubernetes
    uid: 240884a8-50c6-11e7-aae8-001a4a01010c
  subsets:
  - addresses:
    - ip: 10.66.219.113
    ports:
    - name: https
      port: 8443
      protocol: TCP
    - name: dns-tcp
      port: 8053
      protocol: TCP
    - name: dns
      port: 8053
      protocol: UDP
- apiVersion: v1
  kind: Endpoints
  metadata:
    creationTimestamp: 2017-06-14T07:32:51Z
    labels:
      app: registry-console
      createdBy: registry-console-template
      name: registry-console
    name: registry-console
    namespace: default
    resourceVersion: "2858"
    selfLink: /api/v1/namespaces/default/endpoints/registry-console
    uid: ac78a94e-50d3-11e7-aae8-001a4a01010c
  subsets:
  - addresses:
    - ip: 10.128.0.3
      nodeName: node1.ose35.com
      targetRef:
        kind: Pod
        name: registry-console-1-jckp2
        namespace: default
        resourceVersion: "2854"
        uid: d5baeab5-50d3-11e7-aae8-001a4a01010c
    ports:
    - name: registry-console
      port: 9090
      protocol: TCP
kind: List
metadata: {}
resourceVersion: ""
selfLink: ""
""".strip()

OC_GET_NODE = """
apiVersion: v1
items:
- apiVersion: v1
  kind: Node
  metadata:
    annotations:
      volumes.kubernetes.io/controller-managed-attach-detach: "true"
    creationTimestamp: 2018-01-09T02:04:15Z
    labels:
      beta.kubernetes.io/arch: amd64
      beta.kubernetes.io/os: linux
      kubernetes.io/hostname: master37
      openshift-infra: apiserver
    name: master37
    namespace: ""
    resourceVersion: "4414367"
    selfLink: /api/v1/nodes/master37
    uid: 64ce06a1-f4e1-11e7-aa53-001a4a0102af
  spec:
    externalID: master37
    unschedulable: true
  status:
    addresses:
    - address: 10.66.208.248
      type: InternalIP
    - address: master37
      type: Hostname
    allocatable:
      cpu: "8"
      memory: 20587320Ki
      pods: "80"
    capacity:
      cpu: "8"
      memory: 20689720Ki
      pods: "80"
    conditions:
    - lastHeartbeatTime: 2018-02-02T09:46:46Z
      lastTransitionTime: 2018-01-09T02:04:15Z
      message: kubelet has sufficient disk space available
      reason: KubeletHasSufficientDisk
      status: "False"
      type: OutOfDisk
    - lastHeartbeatTime: 2018-02-02T09:46:46Z
      lastTransitionTime: 2018-01-09T02:04:15Z
      message: kubelet has sufficient memory available
      reason: KubeletHasSufficientMemory
      status: "False"
      type: MemoryPressure
    - lastHeartbeatTime: 2018-02-02T09:46:46Z
      lastTransitionTime: 2018-01-09T02:04:15Z
      message: kubelet has no disk pressure
      reason: KubeletHasNoDiskPressure
      status: "False"
      type: DiskPressure
    daemonEndpoints:
      kubeletEndpoint:
        Port: 10250
    images:
    - names:
      - registry.access.redhat.com/openshift3/ose@sha256:c4fe334182030e0878a462998aef2ae48536eb8966c570ddb2710a11b8e9ac82
      - registry.access.redhat.com/openshift3/ose:v3.7
      sizeBytes: 1059063314
    - names:
      - registry.access.redhat.com/openshift3/ose-service-catalog@sha256:e06e609d21f1df6ff396a4d95caade01149e3ee706490b9665708fd95ba84ad0
      - registry.access.redhat.com/openshift3/ose-service-catalog:v3.7
      sizeBytes: 268789707
    - names:
      - registry.access.redhat.com/openshift3/ose-pod@sha256:5b397b3fd1bb98e53e829b8938f853364d3ff85d85f2e011fae3f67757e9cf96
      - registry.access.redhat.com/openshift3/ose-pod:v3.7.14
      sizeBytes: 208847376
    nodeInfo:
      architecture: amd64
      bootID: e998fc70-8d7d-47ac-81c6-72a662c68424
      containerRuntimeVersion: docker://1.12.6
      kernelVersion: 3.10.0-693.11.6.el7.x86_64
      kubeProxyVersion: v1.7.6+a08f5eeb62
      kubeletVersion: v1.7.6+a08f5eeb62
      machineID: 897d22c8a60d434b9b5857ffa80064e4
      operatingSystem: linux
      osImage: Employee SKU
      systemUUID: 897D22C8-A60D-434B-9B58-57FFA80064E4
""".strip()

OC_GET_RC = """
apiVersion: v1
items:
- apiVersion: v1
  kind: ReplicationController
  metadata:
    annotations:
      openshift.io/deployer-pod.name: jenkins-1-deploy
      openshift.io/deployment-config.latest-version: "1"
      openshift.io/deployment-config.name: jenkins
      openshift.io/deployment.phase: Complete
      openshift.io/deployment.replicas: "1"
      openshift.io/deployment.status-reason: image change
    creationTimestamp: 2017-11-28T08:24:39Z
    generation: 2
    labels:
      app: jenkins-ephemeral
      openshift.io/deployment-config.name: jenkins
      template: jenkins-ephemeral-template
    name: jenkins-1
    namespace: ci
    resourceVersion: "9998736"
    selfLink: /api/v1/namespaces/ci/replicationcontrollers/jenkins-1
    uid: 93a12b07-d415-11e7-aef1-001a4a010222
  spec:
    replicas: 1
    selector:
      deployment: jenkins-1
      deploymentconfig: jenkins
      name: jenkins
    template:
      metadata:
        annotations:
          openshift.io/deployment-config.latest-version: "1"
          openshift.io/deployment-config.name: jenkins
          openshift.io/deployment.name: jenkins-1
          openshift.io/generated-by: OpenShiftNewApp
        creationTimestamp: null
        labels:
          app: jenkins-ephemeral
          deployment: jenkins-1
          deploymentconfig: jenkins
          name: jenkins
      spec:
        containers:
        - env:
          - name: JENKINS_PASSWORD
            value: welcome1
          - name: KUBERNETES_MASTER
            value: https://kubernetes.default:443
          - name: KUBERNETES_TRUST_CERTIFICATES
            value: "true"
          - name: JNLP_SERVICE_NAME
            value: jenkins-jnlp
          name: jenkins
          readinessProbe:
            failureThreshold: 3
            httpGet:
              path: /login
              port: 8080
              scheme: HTTP
            initialDelaySeconds: 3
            periodSeconds: 10
            successThreshold: 1
            timeoutSeconds: 3
          resources:
            limits:
              memory: 512Mi
          securityContext:
            capabilities: {}
            privileged: false
          terminationMessagePath: /dev/termination-log
          volumeMounts:
          - mountPath: /var/lib/jenkins
            name: jenkins-data
        dnsPolicy: ClusterFirst
        restartPolicy: Always
        securityContext: {}
        serviceAccount: jenkins
        serviceAccountName: jenkins
        terminationGracePeriodSeconds: 30
        volumes:
        - emptyDir: {}
          name: jenkins-data
  status:
    availableReplicas: 1
    fullyLabeledReplicas: 1
    observedGeneration: 2
    readyReplicas: 1
    replicas: 1
""".strip()

OC_GET_EVENT = """
apiVersion: v1
items:
- apiVersion: v1
  count: 5613
  firstTimestamp: 2018-01-12T02:38:19Z
  involvedObject:
    apiVersion: v1
    fieldPath: spec.containers{busybox}
    kind: Pod
    name: busybox
    namespace: ci
    resourceVersion: "10897114"
    uid: 69f22749-f1da-11e7-989a-001a4a010222
  kind: Event
  lastTimestamp: 2018-02-02T09:49:42Z
  message: pulling image "busybox"
  metadata:
    creationTimestamp: 2018-01-12T02:38:19Z
    name: busybox.1508ef957b1935a4
    namespace: ci
    resourceVersion: "12356614"
    selfLink: /api/v1/namespaces/ci/events/busybox.1508ef957b1935a4
    uid: a648ed7c-f741-11e7-989a-001a4a010222
  reason: Pulling
  source:
    component: kubelet
    host: node2
  type: Normal
- apiVersion: v1
  count: 1
  firstTimestamp: 2018-02-05T01:55:35Z
  involvedObject:
    apiVersion: v1
    kind: Pod
    name: ruby-ex-2-htczn
    namespace: test-ruby
    resourceVersion: "4892941"
    uid: a82ecd0f-0a17-11e8-97fa-001a4a0102af
  kind: Event
  lastTimestamp: 2018-02-05T01:55:35Z
  message: Successfully assigned ruby-ex-2-htczn to node237
  metadata:
    creationTimestamp: 2018-02-05T01:55:35Z
    name: ruby-ex-2-htczn.15104b2e14a7ffc7
    namespace: test-ruby
    resourceVersion: "4892946"
    selfLink: /api/v1/namespaces/test-ruby/events/ruby-ex-2-htczn.15104b2e14a7ffc7
    uid: a831518d-0a17-11e8-97fa-001a4a0102af
  reason: Scheduled
  source:
    component: default-scheduler
  type: Normal
- apiVersion: v1
  count: 1
  firstTimestamp: 2018-02-05T01:55:35Z
  involvedObject:
    apiVersion: v1
    kind: ReplicationController
    name: ruby-ex-2
    namespace: test-ruby
    resourceVersion: "4892939"
    uid: a57f2d52-0a17-11e8-97fa-001a4a0102af
  kind: Event
  lastTimestamp: 2018-02-05T01:55:35Z
  message: 'Created pod: ruby-ex-2-htczn'
  metadata:
    creationTimestamp: 2018-02-05T01:55:35Z
    name: ruby-ex-2.15104b2e13fcd0ca
    namespace: test-ruby
    resourceVersion: "4892945"
    selfLink: /api/v1/namespaces/test-ruby/events/ruby-ex-2.15104b2e13fcd0ca
    uid: a82fc0ed-0a17-11e8-97fa-001a4a0102af
  reason: SuccessfulCreate
  source:
    component: replication-controller
  type: Normal
- apiVersion: v1
  count: 1
  firstTimestamp: 2018-02-05T01:55:31Z
  involvedObject:
    apiVersion: apps.openshift.io
    kind: DeploymentConfig
    name: ruby-ex
    namespace: test-ruby
    resourceVersion: "4892911"
    uid: dafbf582-04a4-11e8-97fa-001a4a0102af
  kind: Event
  lastTimestamp: 2018-02-05T01:55:31Z
  message: Created new replication controller "ruby-ex-2" for version 2
  metadata:
    creationTimestamp: 2018-02-05T01:55:31Z
    name: ruby-ex.15104b2d079675f7
    namespace: test-ruby
    resourceVersion: "4892913"
    selfLink: /api/v1/namespaces/test-ruby/events/ruby-ex.15104b2d079675f7
    uid: a580c543-0a17-11e8-97fa-001a4a0102af
  reason: DeploymentCreated
  source:
    component: deploymentconfig-controller
  type: Normal
kind: List
metadata:
  resourceVersion: ""
  selfLink: ""
""".strip()

OC_GET_EGRESS_NETWORK_POLICY = """
apiVersion: v1
items:
- apiVersion: v1
  kind: EgressNetworkPolicy
  metadata:
    creationTimestamp: 2018-01-29T03:31:59Z
    name: policy-test
    namespace: test-ruby
    resourceVersion: "3649310"
    selfLink: /oapi/v1/namespaces/test-ruby/egressnetworkpolicies/policy-test
    uid: f6b065cc-04a4-11e8-97fa-001a4a0102af
  spec:
    egress:
    - to:
        dnsName: www.baidu.com
      type: Allow
    - to:
        cidrSelector: 0.0.0.0/0
      type: Deny
kind: List
metadata:
  resourceVersion: ""
  selfLink: ""
""".strip()

OC_GET_BUILD = """
apiVersion: v1
items:
- apiVersion: build.openshift.io/v1
  kind: Build
  metadata:
    annotations:
      openshift.io/build-config.name: sample-app
      openshift.io/build.number: "2"
      openshift.io/build.pod-name: sample-app-2-build
    creationTimestamp: 2018-05-08T13:25:35Z
    labels:
      app: sample-app
      buildconfig: sample-app
      openshift.io/build-config.name: sample-app
      openshift.io/build.start-policy: Serial
    name: sample-app-2
    namespace: default
    ownerReferences:
    - apiVersion: build.openshift.io/v1
      controller: true
      kind: BuildConfig
      name: sample-app
      uid: 014c30d6-52c3-11e8-8f2a-001a4a16016f
    resourceVersion: "38277"
    selfLink: /apis/build.openshift.io/v1/namespaces/default/builds/sample-app-2
    uid: 4a656a4e-52c3-11e8-8f2a-001a4a16016f
  spec:
    nodeSelector: null
    output:
      pushSecret:
        name: builder-dockercfg-hwnx4
      to:
        kind: ImageStreamTag
        name: sample-app:latest
    postCommit: {}
    resources: {}
    serviceAccount: builder
    source:
      git:
        uri: https://example.com/mfojtik/sample-app.git
      type: Git
    strategy:
      sourceStrategy:
        from:
          kind: DockerImage
          name: registry.access.example.com/rhscl/ruby-24-rhel7@sha256:da812edc2a0c0c8c28854daeb7629c733dcc5672c33d44f920ea8f1f2d6a058d
      type: Source
    triggeredBy:
    - message: Manually triggered
  status:
    config:
      kind: BuildConfig
      name: sample-app
      namespace: default
    output: {}
    outputDockerImageReference: docker-registry.example.svc:5000/default/sample-app:latest
    phase: Pending
kind: List
metadata:
  resourceVersion: ""
  selfLink: ""
"""


def test_oc_get_pod_yml():
    result = openshift_get.OcGetPod(context_wrap(OC_GET_POD))
    assert result.data['items'][0]['metadata']['annotations']['openshift.io/scc'] == 'anyuid'
    ct = result.data['items'][0]['metadata']['creationTimestamp'].replace(tzinfo=None)
    assert ct == datetime.datetime(2017, 2, 10, 16, 33, 46, tzinfo=None)
    assert result.data['items'][0]['spec']['host'] == 'node2.ose.com'
    assert result.get("items")[0]['spec']['host'] == 'node2.ose.com'
    assert result.pods["router-1-1-w27o2"]["metadata"]["labels"]["deploymentconfig"] == "router-1"


def test_oc_get_service_yml():
    result = openshift_get.OcGetService(context_wrap(OC_GET_SERVICE))
    assert result.data['items'][0]['kind'] == 'Service'
    assert result.data['items'][0]['spec']['clusterIP'] == '172.30.0.1'
    assert result.data['items'][0]['metadata']['name'] == 'kubernetes'
    assert result.data['items'][1]['metadata']['name'] == 'router-1'
    assert result.data['items'][1]['spec']['ports'][0]['port'] == 80
    assert result.data['kind'] == 'List'
    assert result.data['metadata'] == {}
    assert result.get("items")[0]['spec']['clusterIP'] == '172.30.0.1'
    assert "zjj-project" in result.data['items'][1]['metadata']['namespace']
    assert result.services["router-1"]["metadata"]["resourceVersion"] == "1638401"


def test_oc_get_configmap_yml():
    result = openshift_get.OcGetConfigmap(context_wrap(OC_GET_CONFIGMAP))
    assert result.data['items'][0]['kind'] == 'ConfigMap'
    assert result.data['items'][0]['metadata']['name'] == 'node-config-all-in-one'


def test_oc_get_bc_yml():
    result = openshift_get.OcGetBc(context_wrap(OC_GET_BC))
    assert result['items'][0]['kind'] == 'BuildConfig'
    assert result['items'][1]['metadata']['name'] == 'mybank'
    assert result.build_configs['mybank']['status']['lastVersion'] == 11
    assert result.build_configs['tom']['metadata']['namespace'] == 'ci'


def test_oc_get_dc_yml():
    result = openshift_get.OcGetDc(context_wrap(OC_GET_DC))
    assert result.data['items'][0]['kind'] == 'DeploymentConfig'
    assert result.data['items'][0]['metadata']['generation'] == 3
    assert result.get("items")[0]['metadata']['generation'] == 3
    assert result.deployment_configs["router-1"]["metadata"]["namespace"] == "zjj-project"


def test_oc_get_rolebinding_yml():
    result = openshift_get.OcGetRolebinding(context_wrap(OC_GET_ROLEBINDING))
    assert result.data['items'][0]['kind'] == 'RoleBinding'
    assert result.data['items'][0]['metadata']['resourceVersion'] == "11803596"
    assert result.get("items")[0]['metadata']['resourceVersion'] == "11803596"
    assert result.rolebindings["myrole"]["roleRef"]["namespace"] == "foo"


def test_oc_get_project_yml():
    result = openshift_get.OcGetProject(context_wrap(OC_GET_PROJECT))
    assert result.data['items'][0]['kind'] == 'Project'
    assert result.data['items'][0]['metadata']['resourceVersion'] == "11040756"
    assert result.get('items')[0]['metadata']['resourceVersion'] == "11040756"
    assert result.projects["test"]["status"]["phase"] == "Active"


def test_oc_get_role_yml():
    result = openshift_get.OcGetRole(context_wrap(OC_GET_ROLE))
    assert result.data['items'][0]['kind'] == 'Role'
    assert result.data['items'][0]['metadata']['resourceVersion'] == "94"
    assert result.get('items')[0]['metadata']['resourceVersion'] == "94"
    assert result.roles["shared-resource-viewer"]["metadata"]["uid"] == "a10c3f88-6ecc-11e6-83c6-001a4a0101f0"


def test_oc_get_pv_yml():
    result = openshift_get.OcGetPv(context_wrap(OC_GET_PV))
    assert result.data['items'][0]['kind'] == 'PersistentVolume'
    assert result.data['items'][0]['metadata']['name'] == 'registry-volume'
    assert result.get('items')[0]['metadata']['name'] == 'registry-volume'
    assert result.persistent_volumes['registry-volume-zjj']['spec']['capacity']['storage'] == '10Gi'


def test_oc_get_pvc_yml():
    result = openshift_get.OcGetPvc(context_wrap(OC_GET_PVC))
    assert result.data['items'][0]['kind'] == 'PersistentVolumeClaim'
    assert result.data['items'][0]['metadata']['name'] == 'registry-claim'
    assert result.get('items')[0]['metadata']['name'] == 'registry-claim'
    assert result.persistent_volume_claims['registry-claim-test1']['spec']['volumeName'] == 'registry-volume-zjj'


def test_oc_get_endpoints_yml():
    result = openshift_get.OcGetEndPoints(context_wrap(OC_GET_ENDPOINTS))
    assert result.data['items'][0]['kind'] == 'Endpoints'
    assert result.data['items'][0]['metadata']['name'] == 'gluster-cluster'
    assert result.get('items')[0]['metadata']['name'] == 'gluster-cluster'
    assert result.endpoints['kubernetes']['subsets'][0]["addresses"][0]["ip"] == '10.66.219.113'


def test_oc_get_node():
    result = openshift_get.OcGetNode(context_wrap(OC_GET_NODE))
    assert result.data['items'][0]['kind'] == 'Node'
    assert result.data['items'][0]['metadata']['name'] == 'master37'
    assert result.get('items')[0]['metadata']['uid'] == '64ce06a1-f4e1-11e7-aa53-001a4a0102af'
    assert result.nodes['master37']['spec']['unschedulable'] is True


def test_oc_get_rc():
    result = openshift_get.OcGetRc(context_wrap(OC_GET_RC))
    assert result.data['items'][0]['kind'] == 'ReplicationController'
    assert result.data['items'][0]['metadata']['selfLink'] == '/api/v1/namespaces/ci/replicationcontrollers/jenkins-1'
    assert result.get('items')[0]['spec']['replicas'] == 1
    assert result.replication_controllers['jenkins-1']['spec']['selector']['deployment'] == 'jenkins-1'


def test_oc_get_event():
    result = openshift_get.OcGetEvent(context_wrap(OC_GET_EVENT))
    assert result.data['items'][0]['kind'] == 'Event'
    assert result.data['items'][2]['involvedObject']['kind'] == 'ReplicationController'
    assert result.get('items')[1]['message'] == 'Successfully assigned ruby-ex-2-htczn to node237'
    assert result.events['busybox.1508ef957b1935a4']['type'] == 'Normal'


def test_oc_get_egressnetworkpolicy_yml():
    result = openshift_get.OcGetEgressNetworkPolicy(context_wrap(OC_GET_EGRESS_NETWORK_POLICY))
    assert result.data['items'][0]['kind'] == 'EgressNetworkPolicy'
    ct = result.data['items'][0]['metadata']['creationTimestamp'].replace(tzinfo=None)
    assert ct == datetime.datetime(2018, 1, 29, 3, 31, 59, tzinfo=None)
    assert result.get('items')[0]['metadata']['name'] == 'policy-test'
    assert result.egress_network_policies['policy-test']['spec']['egress'][0]['to']['dnsName'] == 'www.baidu.com'


def test_oc_get_services_doc_example():
    env = {
        'OcGetService': openshift_get.OcGetService,
        'setting_dic': openshift_get.OcGetService(context_wrap(OC_GET_SERVICE))
    }
    failed, total = doctest.testmod(openshift_get, globs=env)
    assert failed == 0


def test_oc_get_route():
    result = openshift_get.OcGetRoute(context_wrap(OC_GET_ROUTE))
    assert result.data['items'][0]['kind'] == 'Route'
    assert result.data['items'][0]['metadata']['name'] == 'docker-registry'
    assert result.get('items')[0]['spec']['host'] == 'docker-registry-default.router.default.svc.cluster.local'
    assert result.routes['docker-registry']['spec']['wildcardPolicy'] == 'None'


def test_oc_get_build():
    result = openshift_get.OcGetBuild(context_wrap(OC_GET_BUILD))
    assert result.data['items'][0]['kind'] == 'Build'
    assert result.data['items'][0]['metadata']['annotations']['openshift.io/build.pod-name'] == 'sample-app-2-build'
    assert result.get('items')[0]['status']['phase'] == 'Pending'
    assert result.started_builds['sample-app-2']['status']['config']['kind'] == 'BuildConfig'



"""
redhat-release - File ``/etc/redhat-release``
=============================================

This module provides plugins access to file ``/etc/redhat-release``

Typical content of file ``/etc/redhat-release`` is::

    Red Hat Enterprise Linux Server release 7.2 (Maipo)

This module parses the file content and stores data in the dict ``self.parsed``.
The version info can also be get via ``obj.major`` and ``obj.minor``.
Property ``is_rhel`` and ``is_hypervisor`` specifies the host type.

Examples:
    >>> rh_rls_content = '''
    ... Red Hat Enterprise Linux Server release 7.2 (Maipo)
    ... '''.strip()
    >>> from insights.tests import context_wrap
    >>> shared = {RedhatRelease: RedhatRelease(context_wrap(rh_rls_content))}
    >>> release = shared[RedhatRelease]
    >>> assert release.raw == rh_rls_content
    >>> assert release.major == 7
    >>> assert release.minor == 2
    >>> assert release.version == "7.2"
    >>> assert release.is_rhel
    >>> assert release.product == "Red Hat Enterprise Linux Server"
"""
from .. import Parser, parser
from ..specs import Specs


@parser(Specs.redhat_release)
class RedhatRelease(Parser):
    """Parses the content of file ``/etc/redhat-release``."""

    def parse_content(self, content):
        self.raw = content[0]
        product, _, version_name = [v.strip() for v in content[0].partition("release")]
        version_name_split = [v.strip() for v in version_name.split(None, 1)]
        code_name = (version_name_split[1].strip("()")
                        if len(version_name_split) > 1 else None)
        self.parsed = {
            "product": product,
            "version": version_name_split[0],
            "code_name": code_name
        }

    @property
    def major(self):
        """int: the major version of this OS."""
        return int(self.parsed["version"].split(".")[0])

    @property
    def minor(self):
        """int: the minor version of this OS."""
        s = self.parsed["version"].split("-", 1)[0].split(".")
        if len(s) > 1:
            return int(s[1])

    @property
    def version(self):
        """string: version of this OS."""
        return self.parsed["version"]

    @property
    def is_rhel(self):
        """bool: True if this OS belong to RHEL, else False."""
        return "Red Hat Enterprise Linux" in self.parsed["product"]

    @property
    def product(self):
        """string: product of this OS."""
        return self.parsed["product"]



from insights.components.rhel_version import IsRhel6, IsRhel7, IsRhel8
from insights.combiners.redhat_release import RedHatRelease as RR
from insights.parsers.uname import Uname
from insights.parsers.redhat_release import RedhatRelease
from insights.tests import context_wrap
from insights.core.dr import SkipComponent
import pytest


UNAME = "Linux localhost.localdomain 3.10.0-327.rt56.204.el7.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"

REDHAT_RELEASE1 = """
Red Hat Enterprise Linux Server release 6.7 (Santiago)
""".strip()

REDHAT_RELEASE2 = """
Red Hat Enterprise Linux Server release 7.2 (Maipo)
""".strip()

REDHAT_RELEASE3 = """
Red Hat Enterprise Linux release 7.5-0.14
""".strip()

REDHAT_RELEASE4 = """
Red Hat Enterprise Linux release 8.0 (Ootpa)
""".strip()


# RHEL6 Tests
def test_is_rhel6():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE1))
    rel = RR(None, rr)
    result = IsRhel6(rel)
    assert isinstance(result, IsRhel6)


def test_not_rhel6():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE2))
    rel = RR(None, rr)
    with pytest.raises(SkipComponent) as e:
        IsRhel6(rel)
    assert "Not RHEL6" in str(e)


# RHEL7 Server Tests
def test_is_rhel7s():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE2))
    rel = RR(None, rr)
    result = IsRhel7(rel)
    assert isinstance(result, IsRhel7)


def test_not_rhel7s():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE1))
    rel = RR(None, rr)
    with pytest.raises(SkipComponent) as e:
        IsRhel7(rel)
    assert "Not RHEL7" in str(e)


# RHEL7 Tests
def test_uname_is_rhel7():
    uname = Uname(context_wrap(UNAME))
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE3))
    rel = RR(uname, rr)
    result = IsRhel7(rel)
    assert isinstance(result, IsRhel7)


def test_is_rhel7():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE3))
    rel = RR(None, rr)
    result = IsRhel7(rel)
    assert isinstance(result, IsRhel7)


def test_not_rhel7():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE1))
    rel = RR(None, rr)
    with pytest.raises(SkipComponent) as e:
        IsRhel7(rel)
    assert "Not RHEL7" in str(e)


# RHEL8 Tests
def test_is_rhel8():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE4))
    rel = RR(None, rr)
    result = IsRhel8(rel)
    assert isinstance(result, IsRhel8)


def test_not_rhel8():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE2))
    rel = RR(None, rr)
    with pytest.raises(SkipComponent) as e:
        IsRhel8(rel)
    assert "Not RHEL8" in str(e)



"""
Cobbler settings - ``/etc/cobbler/settings`` file
=================================================

The Cobbler settings file is a **YAML** file and the standard Python ``yaml``
library is used to parse it.

Sample input::

    kernel_options:
        ksdevice: bootif
        lang: ' '
        text: ~

Examples:

    >>> cobbler = shared[CobblerSettings]
    >>> 'kernel_options' in cobbler.data
    True
    >>> cobbler.data['kernel_options']['ksdevice']
    'bootif'

"""

from .. import YAMLParser, parser
from insights.specs import Specs


@parser(Specs.cobbler_settings)
class CobblerSettings(YAMLParser):
    """
    Read the ``/etc/cobbler/settings`` YAML file.
    """
    pass



"""
Parsers for the Corosync Cluster Engine configurations
======================================================


Parsers included in this module are:

CoroSyncConfig - file ``/etc/sysconfig/corosync``
-------------------------------------------------

CorosyncConf - file ``/etc/corosync/corosync.conf``
---------------------------------------------------
"""
import string
from insights.util import deprecated
from insights.core import ConfigParser
from insights.specs import Specs
from .. import SysconfigOptions, parser

from insights.parsr import (EOF, Forward, InSet, LeftCurly, Lift, LineEnd,
        Literal, RightCurly, Many, Number, OneLineComment, PosMarker,
        skip_none, String, QuotedString, WS, WSChar)
from insights.parsr.query import Directive, Entry, Section


@parser(Specs.corosync)
class CoroSyncConfig(SysconfigOptions):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.sysconfig.CorosyncSysconfig` instead.

    This parser reads the ``/etc/sysconfig/corosync`` file. It uses
    the ``SysconfigOptions`` parser class to convert the file into a
    dictionary of options. It also provides the ``options`` property
    as a helper to retrieve the ``COROSYNC_OPTIONS`` variable.

    Sample data::

        # Corosync init script configuration file

        # COROSYNC_INIT_TIMEOUT specifies number of seconds to wait for corosync
        # initialization (default is one minute).
        COROSYNC_INIT_TIMEOUT=60

        # COROSYNC_OPTIONS specifies options passed to corosync command
        # (default is no options).
        # See "man corosync" for detailed descriptions of the options.
        COROSYNC_OPTIONS=""

    Examples:

        >>> 'COROSYNC_OPTIONS' in csconfig.data
        True
        >>> csconfig.options
        ''
    """
    def __init__(self, *args, **kwargs):
        deprecated(CoroSyncConfig, "Import CorosyncSysconfig from insights.parsers.sysconfig instead")
        super(CoroSyncConfig, self).__init__(*args, **kwargs)

    @property
    def options(self):
        """ (str): The value of the ``COROSYNC_OPTIONS`` variable."""
        return self.data.get('COROSYNC_OPTIONS', '')


def parse_doc(f, ctx=None, line_end="\n"):
    def to_entry(name, rest):
        if isinstance(rest, list):
            return Section(name=name.value, children=rest, lineno=name.lineno, src=ctx)
        return Directive(name=name.value, attrs=[rest], lineno=name.lineno, src=ctx)

    Sep = InSet(":=")
    Stmt = Forward()
    Num = Number & (WSChar | LineEnd)
    NULL = Literal("none", value=None)
    Comment = (WS >> OneLineComment("#").map(lambda x: None))
    BeginBlock = (WS >> LeftCurly << WS)
    EndBlock = (WS >> RightCurly << WS)
    Bare = String(set(string.printable) - (set(string.whitespace) | set("#{}'\"")))
    Name = WS >> PosMarker(String(string.ascii_letters + "_" + string.digits)) << WS
    Value = WS >> (Num | NULL | QuotedString | Bare) << WS
    Block = BeginBlock >> Many(Stmt).map(skip_none) << EndBlock
    Stanza = (Lift(to_entry) * Name * (Block | (Sep >> Value))) | Comment
    Stmt <= WS >> Stanza << WS
    Doc = Many(Stmt).map(skip_none)
    Top = Doc + EOF

    return Entry(children=Top(f)[0], src=ctx)


@parser(Specs.corosync_conf)
class CorosyncConf(ConfigParser):
    """Parse the output of the file ``/etc/corosync/corosync.conf`` using
    the ``ConfigParser`` base class. It exposes the corosync
    configuration through the parsr query interface.

    The parameters in the directives are referred from the manpage of
    ``corosync.conf``. See ``man 8 corosync.conf`` for more info.

    Sample content of the file ``/etc/corosync/corosync.conf`` ::

        totem {
            version: 2
            secauth: off
            cluster_name: tripleo_cluster
            transport: udpu
            token: 10000
        }

        nodelist {
            node {
                ring0_addr: overcloud-controller-0
                nodeid: 1
            }

            node {
                ring0_addr: overcloud-controller-1
                nodeid: 2
            }

            node {
                ring0_addr: overcloud-controller-2
                nodeid: 3
            }
        }

        quorum {
            provider: corosync_votequorum
        }

        logging {
            to_logfile: yes
            logfile: /var/log/cluster/corosync.log
            to_syslog: yes
        }


    Example:

        >>> from insights.parsr.query import first, last
        >>> corosync_conf['quorum']['provider'][first].value
        'corosync_votequorum'
        >>> corosync_conf['totem']['token'][first].value
        10000
        >>> corosync_conf['nodelist']['node']['nodeid'][last].value
        3

    """
    def parse_doc(self, content):
        return parse_doc("\n".join(content), ctx=self)
